*DECK ASMDRV
      SUBROUTINE ASMDRV(IPSYS ,IPTRK ,IPMACR,IFTRAK,CDOOR ,IPRNTP,
     >                  NGROUP,NBMIX ,NREGIO,NANI  ,LEAKSW,ITRANC,
     >                  TITRE ,LNORM ,LALBS ,LR2   ,IPHASE,ISCATR,
     >                  KNORM ,CNAME ,
     >                  MATCOD,VOLUME,XSSIGT,XSSIGW)
C
C---------------------------  ASMDRV  ---------------------------------
C
C 1- PROGRAMME STATISTICS:
C
C     NAME     :  ASMDRV
C     USE      :  DRIVER FOR DRAGON ASSEMBLY AND PIJ PHASES
C     MODIFIED :  94-10-17 (AH)
C                 94-07-21 (IP AND GM)
C                 94-06-15 (IP) (PIJK)
C                 91-08-26 (GM)
C                 91-04-02 (RR)
C     AUTHOR   :  G.MARLEAU
C
C 2- ROUTINE PARAMETERS:
C   INPUT
C     IPSYS    : POINTER TO THE PIJ                      I
C     IPTRK    : POINTER TO THE TRACKING                 I
C     IPMACR   : POINTER TO THE MACROLIB                 I
C     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
C     CDOOR    : NAME OF THE PIJ CALCULATION DOOR        C*12
C     IPRNTP   : PRINT OPTION FOR PIJ CALCULATIONS       I
C     NGROUP   : NUMBER OF GROUPS TREATED                I
C     NBMIX    : NUMBER OF MIXTURES CONSIDERED           I
C     NREGIO   : NUMBER OF REGIONS CONSIDERED            I
C     NANI     : NUMBER OF ANISOTROPY ORDERS FOR XS      I
C     LEAKSW   : LEAKAGE SWITCH                          L
C     ITRANC   : TYPE OF TRANSPORT CORRECTIONS           I
C     TITRE    : EXECUTION TITLE                         C*72
C     LNORM    : SWITCH FOR REMOVING LEAKAGE FROM CP     L
C     LALBS    : SWITCH FOR KEEPING THE PIS              L
C     LR2      : =.TRUE. TO STORE INFORMATION REQUIRED   L
C                TO COMPUTE THE L2 NORM OF THE SOLUTION
C     IPHASE   : = -1 FOR ASS AND 0 FOR PIJ 1 FOR PIJK   I
C     ISCATR   : = 0 SCATTERING REDUCED/1 DIRECT         I
C     KNORM    : TYPE OF PIJ NORMALIZATION               I
C     CNAME    : NAME USED TO SAVE ADDITIONAL PIJ DATA   C*12
C     MATCOD   : MIXTURE CODE IN REGION                  I(NREGIO)
C     VOLUME   : VOLUME OF REGION                        R(NREGIO)
C   WORK VECTOR
C     XSSIGT   : GROUP TOTAL CROSS SECTION               R(NBMIX+1)
C     XSSIGW   : WITHIN GROUP SCATTERING                 R(NBMIX+1,
C                                                             NANI)
C
C
C---------------------------   ASMDRV  --------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='ASMDRV')
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE(1)
      COMMON       RBASE
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      IPIJSY,IPIS,IPCSCT,JSIG1
C-----
C INTERFACE VARIABLES
C-----
      INTEGER     IPRNTP,NGROUP,NBMIX,NREGIO,NANI,ITRANC,IGR,ILCMLN,
     >            IPHASE,ISCATR
      INTEGER     MATCOD(*)
      REAL        VOLUME(*),XSSIGT(0:NBMIX),XSSIGW(0:NBMIX,NANI)
C----
C LOCAL VARIABLES
C----
      INTEGER     IPIJK,I,IREGIO,NELPIJ,IL,IPMACR,IPSYS,ITYLCM,IMAT,
     >            IMIX,IPTRK,IFTRAK,KNORM,IOP,NRG,IJKS,KS,NRGI
C
      CHARACTER   CDOOR*12,TITRE*72,CGRPNM*12,CNAME*12,CTEMP*12,
     >            HSMG*130,CMS(3)*1,NDC*4,CM*2
      LOGICAL     LEAKSW,LNORM,LALBS,LR2
C----
C DATA
C-----
      SAVE CMS
      DATA CMS    /'1','2','3'/
C----
C SET DIMENSION FOR DIRECTIONAL CP
C----
      IPIJK=1
      IF(IPHASE .EQ. 1) THEN
        IPIJK=4
      ENDIF
C----
C  CHECK MIXTURE INDICES
C----
      DO 10 I=1,NREGIO
      IF (MATCOD(I).GT.NBMIX) THEN
         WRITE(HSMG,9000) NAMSBR,NBMIX
         CALL XABORT(HSMG)
      ENDIF
   10 CONTINUE
C----
C  READ X-SECTIONS AND COMPUTE TRANSPORT CORRECTED X-SECTIONS
C----
      IF(IPHASE .EQ. -1) THEN
        IF(CDOOR(1:3).NE.'JPM') THEN
          CALL XABORT('ASMDRV: ASSEMBLY ERROR - INVALID DOOR '//CDOOR)
        ENDIF
      ENDIF
C----
C  WRITE CALCULATION OPTIONS
C----
      IF(IPRNTP.GE.1) THEN
        IF(IPHASE .EQ. -1) THEN
          WRITE(IOUT,6200) CDOOR
        ELSE
          WRITE(IOUT,6201) CDOOR
        ENDIF
        IF(ITRANC.GT.0) THEN
          WRITE(IOUT,6100)
        ENDIF
      ENDIF
      IF(IPRNTP.GE.2) THEN
        WRITE(IOUT,6000)
        WRITE(IOUT,6001) (IREGIO,VOLUME(IREGIO),MATCOD(IREGIO),
     >                      IREGIO=1,NREGIO)
      ENDIF
      IF(IPHASE .GE. 0) THEN
C----
C ALLOCATE MEMORY FOR (DOOR)P TO COMPUTE CP MATRICES.
C----
        NELPIJ=NREGIO*(NREGIO+1)/2
        IF( IPRNTP .GT. 1 )THEN
          WRITE(IOUT,6300) (NELPIJ*IPIJK+4*NREGIO*NREGIO)/256
        ENDIF
        CALL SETARA(RBASE,NELPIJ*IPIJK,IPIJSY)
        CALL SETARA(RBASE,NREGIO,IPIS)
      ENDIF
      CALL SETARA(RBASE,NBMIX+1,JSIG1)
      RBASE(JSIG1)=0.0
C----
C  SET CROSS SECTION FOR VOID TO 0.0  (GROUP INDEPENDENT)
C----
      XSSIGT(0)=0.0
      DO 100 IL=1,NANI
        XSSIGW(0,IL)=0.0
 100  CONTINUE
      DO 150 IGR=1,NGROUP
C----
C  READ X-SECTIONS AND COMPUTE TRANSPORT CORRECTED X-SECTIONS
C----
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
        CALL LCMLEN(IPMACR,'TOTAL',ILCMLN,ITYLCM)
        IF(ILCMLN.GT.0) THEN
          CALL LCMGET(IPMACR,'TOTAL',XSSIGT(1))
        ELSE
          CALL LCMLIB(IPMACR)
          CALL XABORT('ASMDRV: READ ERROR ON LCM RECORD= TOTAL')
        ENDIF
        DO 110 IL=1,NANI
          WRITE(CM,'(I2)') IL-1
          CALL LCMLEN(IPMACR,'SIGW'//CM,ILCMLN,ITYLCM)
          IF(ILCMLN.GT.0) THEN
            CALL LCMGET(IPMACR,'SIGW'//CM,XSSIGW(1,IL))
          ELSE
            CALL XDRSET(XSSIGW(1,IL),NBMIX,0.0)
          ENDIF
 110    CONTINUE
        IF(ITRANC.NE.0) THEN
          CALL LCMLEN(IPMACR,'TRANC',ILCMLN,ITYLCM)
          IF(ILCMLN.GT.0) THEN
            CALL LCMGET(IPMACR,'TRANC',RBASE(JSIG1+1))
            DO 125 IMAT=1,NBMIX
              XSSIGT(IMAT)=XSSIGT(IMAT)-RBASE(JSIG1+IMAT)
              XSSIGW(IMAT,1)=XSSIGW(IMAT,1)-RBASE(JSIG1+IMAT)
 125        CONTINUE
          ENDIF
        ENDIF
        CALL LCMPUT(IPSYS,'DRAGON-TXSC',NBMIX+1,2,XSSIGT)
        CALL LCMPUT(IPSYS,'DRAGON-S0XSC',NBMIX+1,2,XSSIGW)
        IF(IPRNTP.GE.3) THEN
          WRITE(IOUT,6002)  IGR
          WRITE(IOUT,6003) (IMIX,XSSIGT(IMIX),XSSIGW(IMIX,1),
     >    IMIX=1,NBMIX)
        ENDIF
        IF(IPHASE .GE. 0) THEN
C----
C  CALL (DOOR)P TO COMPUTE CP MATRICES.
C----
          IF(CDOOR.EQ.'EXCELL') THEN
            CALL EXCELP(IPSYS ,IPTRK ,IFTRAK,IPRNTP,NREGIO,MATCOD,
     >                  VOLUME,KNORM ,NELPIJ,IPIJK ,LEAKSW,TITRE ,
     >                  XSSIGT(1)    ,RBASE(IPIJSY))
          ELSE IF(CDOOR(1:3).EQ.'JPM') THEN
            CALL JPMP  (IPSYS ,IPTRK ,IFTRAK,IPRNTP,NREGIO,MATCOD,
     >                  VOLUME,KNORM ,NELPIJ,IPIJK ,LEAKSW,TITRE ,
     >                  XSSIGT(1)    ,RBASE(IPIJSY))
          ELSE IF(CDOOR.EQ.'SYBIL') THEN
            CALL SYBILP(IPSYS ,IPTRK ,IFTRAK,IPRNTP,NREGIO,MATCOD,
     >                  VOLUME,KNORM ,NELPIJ,IPIJK ,LEAKSW,TITRE ,
     >                  XSSIGT(1)    ,RBASE(IPIJSY))
          ELSE
            CALL XABORT('ASMDRV: '//CDOOR//' IS AN INVALID PIJ DOOR.')
          ENDIF
C----
C  REMOVE LEAKAGE FROM THE CP MATRIX.
C----
          IF(LNORM.OR.LALBS) THEN
            CALL PIJNRM(NREGIO,MATCOD,VOLUME,XSSIGT(1),RBASE(IPIJSY),
     >                  RBASE(IPIS))
            CALL LCMPUT(IPSYS,'DRAGON-PIS',NREGIO,2,RBASE(IPIS))
          ELSE
            CALL PIJCPL(NREGIO,NBMIX ,NELPIJ,MATCOD,VOLUME,XSSIGT,
     >                  RBASE(IPIJSY),RBASE(IPIS)  )
            CALL LCMPUT(IPSYS,'DRAGON-PIS  ',NREGIO,2,RBASE(IPIS))
          ENDIF
          IF(CNAME.NE.' ')THEN
            CALL LCMPUT(IPSYS,CNAME,NELPIJ,2,RBASE(IPIJSY))
            IF(IPIJK .EQ. 4) THEN
              CTEMP=CNAME(1:8)//'PIJK'
              CALL LCMPUT(IPSYS,CTEMP,(IPIJK-1)*NELPIJ,2,
     >        RBASE(IPIJSY+NELPIJ))
            ENDIF
          ENDIF
C----
C  COMPUTE THE SCATTERING MODIFIED CP MATRIX.
C----
          CALL SETARA(RBASE,4*NREGIO*NREGIO,IPCSCT)
          IOP=1
          IF(ISCATR .EQ. 0) THEN
            CALL PIJSMD(IPRNTP,NBMIX,NREGIO,MATCOD,VOLUME,XSSIGW,
     >                  XSSIGT,LEAKSW,RBASE(IPIJSY),RBASE(IPCSCT),IOP)
            CALL XDRSDB(NREGIO*NREGIO,RBASE(IPCSCT),RBASE(IPCSCT),1)
          ELSE
            CALL PIJNOS(NREGIO,VOLUME,RBASE(IPIJSY),RBASE(IPCSCT))
          ENDIF
          CALL LCMPUT(IPSYS,'DRAGON-PCSCT',NREGIO*NREGIO,2,
     >    RBASE(IPCSCT))
C-------
          IF(IPIJK .EQ. 4)THEN
            IOP=4
            NRG=NREGIO*NREGIO
            NRGI=3*NRG
            IF(ISCATR .EQ. 0)THEN
              DO 200 IJKS=1,3
                WRITE(NDC,'(2HDC,I2)') IJKS
                CALL LCMLEN(IPMACR,NDC,ILCMLN,ITYLCM)
                IF(ILCMLN.GT.0) THEN
                  CALL LCMGET(IPMACR,NDC,RBASE(JSIG1+1))
                ELSE
                  CALL XDRSET(RBASE(JSIG1+1),NBMIX,0.0)
                ENDIF
                DO 135 IMAT=1,NBMIX
                  RBASE(JSIG1+IMAT)=XSSIGW(IMAT,2)+RBASE(JSIG1+IMAT)
 135            CONTINUE
                KS=IPIJSY+NELPIJ*IJKS
                CALL PIJSMD(IPRNTP,NBMIX,NREGIO,MATCOD,VOLUME,
     >                      RBASE(JSIG1),XSSIGT,LEAKSW,
     >                      RBASE(KS),RBASE(IPCSCT),IOP)
                CALL XDRSDB(NREGIO*NREGIO,RBASE(IPCSCT),
     >                      RBASE(IPCSCT),1)
                CALL LCMPUT(IPSYS,'DRAGON'//CMS(IJKS)//'PCSCT',NRG,2,
     >                      RBASE(IPCSCT))
 200          CONTINUE
            ELSE
              DO 300 IJKS=1,3
                KS=IPIJSY+NELPIJ*IJKS
                CALL PIJNOS(NREGIO,VOLUME,RBASE(KS),RBASE(IPCSCT))
                CALL LCMPUT(IPSYS,'DRAGON'//CMS(IJKS)//'PCSCT',NRG,2,
     >                      RBASE(IPCSCT))
 300          CONTINUE
            ENDIF
          ENDIF
          CALL RLSARA(RBASE(IPCSCT))
        ELSE
C----
C  CALL (DOOR)A TO ASSEMBLE RESPONSE MATRICES
C----
          IF(CDOOR(1:3).EQ.'JPM') THEN
            CALL JPMA(IPSYS,IPTRK,IFTRAK,IPRNTP,LR2,NREGIO,NBMIX,NANI,
     >                MATCOD,VOLUME,KNORM,XSSIGT,XSSIGW,LEAKSW,TITRE)
          ELSE
            CALL XABORT('ASMDRV: '//CDOOR//' IS AN INVALID ASM DOOR.')
          ENDIF
        ENDIF
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
        CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
 150  CONTINUE
      CALL RLSARA(RBASE(JSIG1))
      IF(IPHASE .GE. 0) THEN
C----
C  RELEASE MEMORY FOR (DOOR)P TO COMPUTE CP MATRICES.
C----
        CALL RLSARA(RBASE(IPIS))
        CALL RLSARA(RBASE(IPIJSY))
      ENDIF
      IF(LNORM.OR.LALBS) LEAKSW=.FALSE.
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(//30X,' EDITION REGION/VOLUME/MIXTURE '//
     >3(5X,'REGION',5X,'VOLUME  ',5X,'MIXTURE')/)
 6001 FORMAT(1P,3(5X,I4,4X,E12.5,4X,I4,4X))
 6002 FORMAT(//30X,' G R O U P : ',I5//
     >30X,' TOTAL MACROSCOPIC CROSS SECTIONS PER MIXTURE '/)
 6003 FORMAT(3(1X,'MIXTURE',4X,'TOTAL',11X,'SIGW',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
 6100 FORMAT(//' USE TRANSPORT CORRECTED CROSS-SECTIONS')
 6200 FORMAT(//' COMPUTATION OF DRAGON RESPONSE MATRICES BY DOOR =',
     >3X,A12)
 6201 FORMAT(//' COMPUTATION OF DRAGON COMPLETE CP BY DOOR =',
     >3X,A12)
 6300 FORMAT(' *** SPACE REQUIRED FOR CP EVALUATION = ',I10,' K ***')
C----
C  ABORT FORMAT
C----
 9000 FORMAT(1X,A6,': INVALID VALUE OF NBMIX (',I5,').')
      END
