*DECK EVOBLD
      SUBROUTINE EVOBLD(IPDEPL,IPLIB,IMPX,INDREC,NBISO,NGROUP,NPROC,
     >                  ISONAM,AWRISO,NCOMB,NDEPL,NDFI,NDFP,NHEAVY,
     >                  NLIGHT,NOTHER,NBXR,EPS1,EPS2,EXPMAX,H1,ITYPE,
     >                  INR,FIT,XT,ITIXS,DELTA,FNORM,JM,YDPL,FLX,VX,
     >                  IDR,RER,KPAR,BPAR,YIELD,MU1,IMA,SIG,
     >                  SFIS,LP,QF)
C
C-----------------------------------------------------------------------
C
C COMPUTE THE DEPLETION REACTION RATES, PERFORM FLUX NORMALIZATION
C AND CALL EVODPL TO SOLVE THE DEPLETION SYSTEM FOR EACH DEPLETING
C MIXTURE BETWEEN TIMES XT(1) AND XT(2).
C
C INPUT AND OUTPUT PARAMETERS:
C  IPDEPL  : POINTER TO THE DEPLETION HISTORY
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C  IMPX    : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
C  INDREC  : =1: BEGINNING OF DEPLETION; =2: ELSE.
C  NBISO   : NUMBER OF ISOTOPES/MATERIALS INCLUDING NON-DEPLETING ONES.
C  NGROUP  : NUMBER OF ENERGY GROUPS.
C  NPROC   : NUMBER OF MICROSCOPIC XS TO PROCESS .
C  ISONAM  : ALIAS NAMES OF ISOTOPES. ISONAM(3,NBISO)
C  AWRISO  : ATOMIC WEIGHT ISOTOPES. AWRISO(NBISO)
C  NCOMB   : NUMBER OF DEPLETING MIXTURES.
C  NDEPL   : NUMBER OF DEPLETING NUCLIDES.
C  NDFI    : NUMBER OF DIRECT FISSILE ISOTOPES.
C  NDFP    : NUMBER OF DIRECT FISSION PRODUCTS.
C  NHEAVY  : NUMBER OF HEAVY ISOTOPES.
C  NLIGHT  : NUMBER OF LIGHT ISOTOPES.
C  NOTHER  : NUMBER OF OTHER ISOTOPES.
C  NBXR    : MAXIMUM NUMBER OF PARENT REACTION.
C  EPS1    : REQUIRED ACCURACY FOR ODE SOLVER.
C  EPS2    : REQUIRED ACCURACY FOR CONSTANT POWER ITERATIONS.
C  EXPMAX  : ISOTOPE SATURATING LIMIT.
C  H1      : GUESSED FIRST STEPSIZE.
C  ITYPE   : =1 FIFTH-ORDER RUNGE-KUTTA METHOD.
C            =2 FOURTH-ORDER KAPS-RENTROP METHOD.
C  INR     : =0: OUT-OF-CORE DEPLETION.
C            =1: CONSTANT FLUX DEPLETION.
C            =2: CONSTANT POWER DEPLETION.
C  FIT     : FLUX NORMALIZATION FACTOR. N/CM**2/S IF INR=1 ;
C            MW/TONNE OF INITIAL HEAVY ELEMENTS IF INR=2.
C  XT      : INITIAL AND FINAL TIME FOR THE DEPLETION. XT(2)
C  ITIXS   : FLAG FOR TIME DEPENDENT XS (=0 ON/=1 OFF)
C  DELTA   : DELTA(1) : INCREMENT IN FUEL BURNUP FOR THIS STAGE ;
C            DELTA(2) : INCREMENT IN NEUTRON EXPOSURE FOR THIS STAGE.
C  FNORM   : DRAGON FLUX NORMALIZATION FACTOR. DIMENSION FNORM(2)
C  JM      : POSITION IN ISOTOPE LIST OF EACH DEPLETING NUCLIDE.
C            DIMENSION JM(NDEPL,NCOMB)
C  YDPL    : INITIAL/FINAL NUMBER DENSITY OF ISOTOPE IN THE DEPLETION
C            CHAIN. YDPL(NDEPL+1,2,ICMB) IS THE STAGE BURNUP INCREMENT
C            IN REGION ICMB. DIMENSION YDPL(NDEPL+1,2,NCOMB)
C  FLX     : INITIAL/FINAL VOLUME AVERAGED NEUTRON FLUXES.
C            DIMENSION FLX(NGROUP,2,NCOMB)
C  VX      : VOLUMES OF THE DEPLITING MIXTURES. DIMENSION VX(NCOMB)
C  IDR     : REACTION IDENTIFIER. IDR(MAXR,NDEPL)
C  RER     : REACTION CONSTANT. RER(MAXR,NDEPL)
C  KPAR    : REDUCED REACTION TYPE MATRIX. KPAR(NBXR,NDEPL)
C  BPAR    : REDUCED BRANCHING RATIO MATRIX. BPAR(NBXR,NDEPL)
C  YIELD   : FISSION YIELD. YIELD(NDFI,NDFP)
C
C SCRATCH STORAGE:
C  MU1     : DIMENSION MU1(NDEPL+1)
C  IMA     : DIMENSION IMA(NDEPL+1)
C  SIG     : DIMENSION SIG(NDEPL,5,2,NCOMB)
C  SFIS    : DIMENSION SFIS(NHEAVY,2)
C  LP      : DIMENSION LP(NDEPL)
C  QF      : QF(NHEAVY)
C
C-------------------------------------- AUTHOR: A. HEBERT  17/07/90 ----
C-------------------------------------- MODIF : G. MARLEAU 97/02/13 ----
C-------------------------------------- MODIF : G. MARLEAU 98/11/10 ----
C                             CORRECT FOR HEAVY FISSION PRODUCT (WIMSD4)
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER           IOUT,MAXR,MXTIMS,KFISSI,NSTATE
      REAL              ANEUT
      PARAMETER        (IOUT=6,MAXR=5,MXTIMS=2,ANEUT=1.008665,
     >                  KFISSI=6,NSTATE=20)
C----
C MEMORY ALLOCATION
C----
      INTEGER          IBASE(1)
      REAL             RBASE(1)
      COMMON           RBASE
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          INDPRO,ITYPRO,JXSREC,IBLOC,IIPERM,IDIAG,IADPL,
     >                 IBDPL,IKFIS,IYSF,IKSAT,IYST1,IYSAT,IMU12,IIMA2,
     >                 IADPL2,IBDPL2,IYSF2,IKFIS2
C----
C INTERFACE AND LOCAL  VARIABLES
C----
      INTEGER           IPDEPL,IPLIB,IMPX,INDREC,NBISO,NGROUP,NPROC,
     >                  NCOMB,NDEPL,NDFI,NDFP,NHEAVY,
     >                  NLIGHT,NOTHER,NBXR,ITYPE,INR,ITIXS,
     >                  ICMB,IG,IH,ISO,IP,IQ,IS,NITER,NDEPLT,ITER,
     >                  NILT,NID,IDEPL,IDFI,NIH,NIF,NIL,MXDADP
      INTEGER           ISONAM(3,NBISO),JM(NDEPL,NCOMB),IDR(MAXR,NDEPL),
     >                  KPAR(NBXR,NDEPL),MU1(NDEPL+1),IMA(NDEPL+1),
     >                  LP(NDEPL)
C
      REAL              EPS1,EPS2,EXPMAX,H1,FIT,VTOT,VPHI,XX,XPHIP,
     >                  XNREF,XNNEW,ERROR,FACT
      REAL              YDPL(NDEPL+1,2,NCOMB),FLX(NGROUP,2,NCOMB),
     >                  XT(2),DELTA(2),FNORM(2),AWRISO(NBISO),
     >                  VX(NCOMB),RER(MAXR,NDEPL),BPAR(NBXR,NDEPL),
     >                  YIELD(NDFI,NDFP),SIG(NDEPL,5,2,NCOMB),
     >                  SFIS(NHEAVY,2),QF(NHEAVY),PHI(MXTIMS),
     >                  FUELDN(MXTIMS)
      DOUBLE PRECISION  GAR,FUELDD
      INTEGER           ICPFD,ILNLCM,ITYLCM
      LOGICAL           LCOOL
C----
C  INITIALIZE DATA.
C----
      LCOOL=(INR.EQ.0).OR.(FIT.EQ.0.0)
      IF(.NOT.LCOOL) THEN
        LCOOL=.TRUE.
        DO 100 ICMB=1,NCOMB
          DO 101 IG=1,NGROUP
            LCOOL=LCOOL.AND.(FLX(IG,1,ICMB).EQ.0.).AND.
     >                      (FLX(IG,2,ICMB).EQ.0.)
 101      CONTINUE
 100    CONTINUE
      ENDIF
C----
C  FIND INTEGRATED VOLUME
C----
      VTOT=0.0
      DO 110 ICMB=1,NCOMB
        VTOT=VTOT+VX(ICMB)
 110  CONTINUE
C----
C  FIND HEAVY ELEMENT MASS IN FUEL
C----
      ICPFD=0
      IF(INDREC .NE. 1) THEN
*----
*  Locate old fuel density on IPDEPL
*  and read it if present
*----
        CALL LCMLEN(IPDEPL,'FUELDEN-INIT',ILNLCM,ITYLCM)
        IF(ILNLCM .GT. 0) THEN 
          CALL LCMGET(IPDEPL,'FUELDEN-INIT',FUELDN)
          ICPFD=1
        ENDIF
      ENDIF
*----
*  Compute fuel density and store on IPDEPL
*  if not already available
*----
      IF(ICPFD .EQ. 0) THEN
        FUELDD=0.0D0
        DO 120 ICMB=1,NCOMB
          DO 121 IH=1,NHEAVY
            ISO=JM(IH,ICMB)
            FUELDD=FUELDD+DBLE(AWRISO(ISO)*YDPL(IH,1,ICMB)*VX(ICMB))
 121      CONTINUE
 120    CONTINUE
        FUELDN(1)=FUELDD*ANEUT/(0.6023*VTOT)
        FUELDN(2)=FUELDN(1)*VTOT
        CALL LCMPUT(IPDEPL,'FUELDEN-INIT',2,2,FUELDN)
      ENDIF
      IF(IMPX.GE.1) WRITE(IOUT,6000) FUELDN(1),VTOT,FUELDN(2)
C----
C  INITIALIZE REACTION RATES
C----
      CALL XDRSET(SIG,5*NCOMB*MXTIMS*NDEPL,0.0)
      CALL XDRSET(SFIS,2*NHEAVY,0.0)
      IF(.NOT.LCOOL) THEN
C----
C  NORMALIZE AVERAGED FLUX TO 1 (INTEGRATED FLUX TO VTOT)
C----
        DO 130 IP=1,MXTIMS
          VPHI=0.0
          DO 131 ICMB=1,NCOMB
            DO 132 IG=1,NGROUP
              VPHI=VPHI+VX(ICMB)*FLX(IG,IP,ICMB)
 132        CONTINUE
 131      CONTINUE
          VPHI=VTOT/VPHI
          DO 133 ICMB=1,NCOMB
            DO 134 IG=1,NGROUP
              FLX(IG,IP,ICMB)=FLX(IG,IP,ICMB)*VPHI
 134        CONTINUE
 133      CONTINUE
          FNORM(IP)=FNORM(IP)*VPHI
 130    CONTINUE
      ENDIF
C----
C  COMPUTE REACTION RATES FOR DEPLETION WITH NON-ZERO FLUX
C----
      CALL SETARA(IBASE,NPROC,INDPRO)
      CALL SETARA(IBASE,NPROC,ITYPRO)
      CALL SETARA(RBASE,NGROUP*NPROC,JXSREC)
      CALL SETARA(IBASE,2*NCOMB,IBLOC)
      CALL EVORRE(IPLIB,NBISO,NGROUP,ISONAM,NCOMB,NDEPL,NDFI,
     >            NHEAVY,XT,ITIXS,JM,YDPL,LCOOL,FLX,VX,IDR,RER,
     >            VTOT,SIG,SFIS,QF,RBASE(IBLOC),NPROC,IBASE(INDPRO),
     >            IBASE(ITYPRO),RBASE(JXSREC))
      CALL RLSARA(IBASE(IBLOC))
      CALL RLSARA(RBASE(JXSREC))
      CALL RLSARA(IBASE(ITYPRO))
      CALL RLSARA(IBASE(INDPRO))
      IF(.NOT.LCOOL) THEN
C----
C  FIXED POWER AND FLUX LEVEL DEPLETION
C  FIND INITIAL AND FINAL FLUX NORMALIZATION
C----
        IF(INR       .EQ. 2 .AND.
     >     FUELDN(1) .EQ. 0.0    )
     >    CALL XABORT('EVOBLD: POWER BURNUP REQUIRES '//
     >                'NON ZERO FUEL DENSITY')
        DO 140 IP=1,MXTIMS
          IF(INR.EQ.2) THEN
            XX=0.0
            DO 141 IDFI=1,NHEAVY
              IF(MOD(IDR(5,IDFI),10).EQ.KFISSI)
     >          XX=XX+SFIS(IDFI,IP)*QF(IDFI)
 141        CONTINUE
            PHI(IP)=FIT*FUELDN(1)/(1.602*XX)
          ELSE
            PHI(IP)=FIT*1.E-13
          ENDIF
          FNORM(IP)=FNORM(IP)*PHI(IP)
          DO 142 ICMB=1,NCOMB
            DO 143 IG=1,NGROUP
              FLX(IG,IP,ICMB)=FLX(IG,IP,ICMB)*PHI(IP)
 143        CONTINUE
            XPHIP=PHI(IP)*1.0E-3
            DO 144 IQ=2,5
              DO 145 IS=1,NDEPL
                 SIG(IS,IQ,IP,ICMB)=SIG(IS,IQ,IP,ICMB)*XPHIP
 145          CONTINUE
 144        CONTINUE
 142      CONTINUE
 140    CONTINUE
        IF(IMPX.GE.5) WRITE(IOUT,6010) (PHI(IP)*1.E13,IP=1,MXTIMS)
      ENDIF
C----
C  FOR FIXED POWER,
C    FIRST SOLVE DEPLETION EQUATION WITHOUT FP - NITER=10
C    AND APPROXIMATE FINAL FLUX
C    THEN SOLVE FULL DEPLETION EQUATION WITH FP - NITER=0
C  FOR FIXED FLUX AND OUT-OF-CORE
C    SOLVE FULL DEPLETION EQUATION WITH FP - NITER=0
C----
      IF(LCOOL.OR.INR.EQ.1) THEN
        NITER=0
        NDEPLT=NDEPL
      ELSE
        NITER=10
        NDEPLT=NHEAVY
      ENDIF
      CALL XDRSET(LP,NDEPL,0)
      ITER=0
      NILT=NDEPLT-NHEAVY
C----
C   BEGIN ITERATION LOOP
C----
 150  CONTINUE
      ITER=ITER+1
C----
C  SOLVE DEPLETION EQUATIONS FOR EACH BURNING REGION
C----
      DO 160 ICMB=1,NCOMB
C----
C  DETERMINE THE ROW AND COLUMN PROFILE OF THE ADPL MATRIX AND
C  COMPUTE NID, THE NUMBER OF DEPLETING ISOTOPES IN REGION ICMB.
C----
        NID=0
        NIH=0
        DO 161 IDEPL=1,NDEPLT
          LP(IDEPL)=0
          IF(JM(IDEPL,ICMB).GT.0) THEN
            NID=NID+1
            LP(IDEPL)=NID
            IF(IDEPL.LE.NHEAVY) NIH=NID
          ENDIF
          MU1(IDEPL)=1
          IMA(IDEPL)=1
 161    CONTINUE
        IF(NID.GT.0) THEN
C----
C  DETERMINE THE ROW AND COLUMN PROFILE OF THE ADPL MATRIX
C  98/11/10 - CORRECT FOR HEAVY FISSION PRODUCT (WIMSD4)
C             TO SIMULATE N->2N REACTION
C----
          CALL SETARA(IBASE,NDEPLT,IIPERM)
          CALL SETARA(RBASE,NDEPLT,IDIAG)
          CALL EVORCP(IMPX,NCOMB,NDEPL,NDFI,NDFP,NBXR,EXPMAX,XT,IDR,
     >                RER,KPAR,YIELD,SIG,LCOOL,NDEPLT,NID,NIF,LP,
     >                MU1,IMA,IBASE(IIPERM),RBASE(IDIAG))
          CALL RLSARA(RBASE(IDIAG))
          CALL RLSARA(IBASE(IIPERM))
          NIL=NID-NIH
          MXDADP=2*IMA(NID+1)
          CALL SETARA(RBASE,MXDADP*2,IADPL)
          CALL SETARA(RBASE,(NID+1)*2,IBDPL)
          CALL SETARA(IBASE,NIF,IKFIS)
          CALL SETARA(RBASE,NIF*(NIL+1)*2,IYSF)
C----
C  PREPARE DEPLETION MATRICES
C  STORE IN SPARSE STORAGE MODE FOR A SINGLE
C  DEPLETING MIXTURE.
C  98/11/10 - CORRECT FOR HEAVY FISSION PRODUCT (WIMSD4)
C             TO SIMULATE N->2N REACTION
C----
          CALL EVOPRE(NDEPL,NDFI,NDFP,NBXR,NHEAVY,IDR,
     >                KPAR,BPAR,YIELD,SIG(1,1,1,ICMB),QF,LCOOL,NDEPLT,
     >                NID,NIF,NIH,NIL,LP,MU1,IMA,MXDADP,RBASE(IADPL),
     >                RBASE(IBDPL),IBASE(IKFIS),RBASE(IYSF))
C----
C  SET FINAL DENSITY TO INITIAL DENSITY FOR DEPLETING ISOTOPES
C----
          DO 162  IDEPL=1,NDEPL
            IF((LP(IDEPL).GT.0).AND.(LP(IDEPL).LE.NID)) THEN
              YDPL(LP(IDEPL),2,ICMB)=YDPL(IDEPL,1,ICMB)
            ENDIF
 162      CONTINUE
          YDPL(NID+1,2,ICMB)=0.0
          CALL SETARA(IBASE,NID+1,IKSAT)
          CALL SETARA(IBASE,NID+1,IIPERM)
          CALL SETARA(RBASE,NID+1,IYST1)
          CALL SETARA(RBASE,NID+1,IYSAT)
          CALL SETARA(IBASE,NID+1,IMU12)
          CALL SETARA(IBASE,NID+1,IIMA2)
          CALL SETARA(RBASE,2*MXDADP,IADPL2)
          CALL SETARA(RBASE,2*(NID+1),IBDPL2)
          CALL SETARA(RBASE,2*NIF*(NIL+1),IYSF2)
          CALL SETARA(IBASE,NIF,IKFIS2)
C----
C  SOLVE THE DEPLETION EQUATIONS BETWEEN TIMES XT(1) AND XT(2).
C----
          CALL EVODPL(IMPX,YDPL(1,2,ICMB),NID+1,XT,EPS1,EXPMAX,
     >                H1,ITYPE,MU1,IMA,MXDADP,NIL+1,NIF,
     >                IBASE(IKFIS),RBASE(IYSF),RBASE(IADPL),
     >                RBASE(IBDPL),IBASE(IKSAT),IBASE(IIPERM),
     >                RBASE(IYST1),RBASE(IYSAT),IBASE(IMU12),
     >                IBASE(IIMA2),RBASE(IADPL2),RBASE(IBDPL2),
     >                RBASE(IYSF2),IBASE(IKFIS2))
          CALL RLSARA(IBASE(IKFIS2))
          CALL RLSARA(RBASE(IYSF2))
          CALL RLSARA(RBASE(IBDPL2))
          CALL RLSARA(RBASE(IADPL2))
          CALL RLSARA(IBASE(IIMA2))
          CALL RLSARA(IBASE(IMU12))
          CALL RLSARA(RBASE(IYSAT))
          CALL RLSARA(RBASE(IYST1))
          CALL RLSARA(IBASE(IIPERM))
          CALL RLSARA(IBASE(IKSAT))
          YDPL(NDEPL+1,2,ICMB)=YDPL(NID+1,2,ICMB)
          DO 163  IDEPL=NDEPL,1,-1
            IF((LP(IDEPL).GT.0).AND.(LP(IDEPL).LE.NID)) THEN
              YDPL(IDEPL,2,ICMB)=YDPL(LP(IDEPL),2,ICMB)
            ELSE
              YDPL(IDEPL,2,ICMB)=0.0
            ENDIF
 163      CONTINUE
          CALL RLSARA(RBASE(IYSF))
          CALL RLSARA(IBASE(IKFIS))
          CALL RLSARA(RBASE(IBDPL))
          CALL RLSARA(RBASE(IADPL))
        ENDIF
        IF(.NOT.LCOOL.AND.NITER.EQ.0) THEN
          DELTA(1)=DELTA(1)+YDPL(NDEPL+1,2,ICMB)*VX(ICMB)
        ENDIF
 160  CONTINUE
      IF(NITER.EQ.0) GO TO 155
C----
C  FIXED POWER AND FLUX LEVEL DEPLETION
C  FIND REEVALUATE FINAL FLUX NORMALIZATION
C----
      XNREF=PHI(2)
      XPHIP=1.0E3/(VTOT*XNREF)
      XX=0.0
      DO 170 IDFI=1,NHEAVY
        IF(MOD(IDR(5,IDFI),10).EQ.KFISSI) THEN
          GAR=0.0D0
          DO 171 ICMB=1,NCOMB
            GAR=GAR+VX(ICMB)*YDPL(IDFI,2,ICMB)*SIG(IDFI,2,2,ICMB)
 171      CONTINUE
          SFIS(IDFI,2)=XPHIP*GAR
          XX=XX+SFIS(IDFI,2)*QF(IDFI)
        ELSE
          SFIS(IDFI,2)=0.0
        ENDIF
 170  CONTINUE
      XNNEW=FIT*FUELDN(1)/(1.602*XX)
      ERROR=ABS(XNNEW-XNREF)/ABS(XNNEW)
      PHI(2)=XNNEW
      XNNEW=XNNEW/XNREF
      FNORM(2)=FNORM(2)*XNNEW
      IF(IMPX.GE.5) WRITE(IOUT,6011) ITER,ERROR
      DO 190 ICMB=1,NCOMB
        DO 191 IG=1,NGROUP
          FLX(IG,2,ICMB)=FLX(IG,2,ICMB)*XNNEW
 191    CONTINUE
        DO 192 IQ=2,5
          DO 193 IS=1,NDEPL
            SIG(IS,IQ,2,ICMB)=SIG(IS,IQ,2,ICMB)*XNNEW
 193      CONTINUE
 192    CONTINUE
 190  CONTINUE
C----
C  CHECK IF FINAL FLUX CONVERGED
C----
      IF(ERROR.LT.EPS2) THEN
        NITER=0
        NDEPLT=NDEPL
        NILT=NDEPLT-NHEAVY
      ELSE IF(ITER.GE.NITER) THEN
        CALL XABORT('EVOBLD: UNABLE TO CONVERGE.')
        XNREF=PHI(2)
      ENDIF
      IF(IMPX.GE.5) WRITE(IOUT,6010) (PHI(IP)*1.E13,IP=1,MXTIMS)
      GO TO 150
C----
C  SOLUTION COMPLETED
C----
 155  CONTINUE
      IF(.NOT.LCOOL) THEN
        IF(FUELDN(1) .EQ. 0.0) THEN
          FACT=0.0
        ELSE
          FACT=1602.0/FUELDN(2)
        ENDIF
        DELTA(1)=0.0
        DO 200 ICMB=1,NCOMB
          DELTA(1)=DELTA(1)+YDPL(NDEPL+1,2,ICMB)*VX(ICMB)
 200    CONTINUE
        DELTA(1)=DELTA(1)*FACT
        DELTA(2)=0.5*(PHI(1)+PHI(2))*(XT(2)-XT(1))
        IF(IMPX.GE.1) WRITE(IOUT,6001) DELTA(1)/8.64E-4,DELTA(2)
      ELSE
        DELTA(1)=0.0
        DELTA(2)=0.0
      ENDIF
C----
C  FORMAT
C----
 6000 FORMAT(' -> FUEL PROPERTIES',1P/
     >       '    FUEL INITIAL DENSITY  = ',E14.6,' g/cm**3'/
     >       '    FUEL TOTAL VOLUME     = ',E14.6,' cm**3'/
     >       '    FUEL INITIAL MASS     = ',E14.6,' g')
 6001 FORMAT(' -> BURNUP INCREMENT',1P/
     >       '    FUEL BURNUP INCREMENT      = ',E14.6,' MW*D/TON'/
     >       '    NEUTRON EXPOSURE INCREMENT = ',E14.6,' N/KB')
 6010 FORMAT(' -> VALUES OF AVERAGED FLUX',1P/
     >       '    INITIAL  = ',E14.6,' N/CM**2/S'/
     >       '    FINAL    = ',E14.6,' N/CM**2/S')
 6011 FORMAT(' -> ITERATION ON FINAL POWER NB.',I3,3X,
     >       'ERROR=',1P,E14.6)
      RETURN
      END
