*DECK HSTGMA
      SUBROUTINE HSTGMA(IPMAP,  IPRINT, NCHA,   NBUN,  DELTAT,
     >                  POWER,  IREFUS, REFUT,  NBFUEL)
*
*----------
*
*Purpose:
* To read from the MAP data structure the power and 
* burnup distribution for each cell as well as the refueling 
* option for each channel.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): G. Marleau
*
*Parameters: input
* IPMAP   address of the \dds{map} data structure.
* IPRINT  print level.
* NCHA    number of fuel channels.                   
* NBUN    number of bundles per channel.            
* IPMAP   pointer to the MAP data structure
* IPRINT  print level.
* NCHA    number of fuel channels.
* NBUN    number of bundles per channels.
*
*Parameters: input/output
* DELTAT  last character string read.
* POWER   burnup power for each fuel bundle in each channel.
* IREFUS  refueling strategy for each channel.
* REFUT   refueling time for each channel.
* NBFUEL  number of fueled channels.            
* DELTAT  next time steps for burnup.
* POWER   values of local powers.
* IREFUS  fuels shift for each channel. 
*         A channel is refueled using a NBS bundle 
*         shift procedure if IREFUS(I)=NBS. 
*         In the case where NBS $>$ 0,
*         bundles 1 to NBUN-NBS are displaced to position NBS+1 to
*         NBUN while locations 1 to NBS are filled with new fuel. 
*         In the case where NBS $<$ 0,
*         bundles -NBS+1 to NBUN are displaced to position 1 to
*         NBUN+NBS while locations NBUN+NBS+1 to NBUN are filled 
*         with new fuel.
* REFUT   channel refueling time.
* NBFUEL  number of fueled channels 
*
*----------
*
      IMPLICIT         NONE
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER          IPMAP,IPRINT,NCHA,NBUN
      REAL             DELTAT
      REAL             POWER(NCHA,NBUN)
      INTEGER          IREFUS(NCHA)
      REAL             REFUT(NCHA)
      INTEGER          NBFUEL
*----
*  LOCAL PARAMETERS
*----
      INTEGER          IOUT,NTC,ILCMUP,ILCMDN
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NTC=3,ILCMUP=1,ILCMDN=2,
     >                 NAMSBR='HSTGMA')
*----
*  LOCAL VARIABLES
*----                  
      INTEGER          ILCMLN,ILCMTY
      INTEGER          IC 
*----
*  Read DEPL-TIME
*----
      CALL LCMLEN(IPMAP,'DEPL-TIME   ',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0) THEN 
        IF(ILCMLN .GT. 1) CALL XABORT(NAMSBR//
     >  ': Space to store next time step is too small')
        CALL LCMGET(IPMAP,'DEPL-TIME   ',DELTAT)
      ENDIF
*----
*  Read POWER-BUN
*----
      CALL LCMLEN(IPMAP,'POWER-BUN   ',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0) THEN 
        IF(ILCMLN .GT. NCHA*NBUN) CALL XABORT(NAMSBR//
     >  ': Space to store power is too small')
        CALL LCMGET(IPMAP,'POWER-BUN   ',POWER)
      ENDIF
*----
*  Read refueling scheme
*----
      CALL LCMLEN(IPMAP,'REFUEL-SCH  ',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0) THEN 
        IF(ILCMLN .GT. NCHA) CALL XABORT(NAMSBR//
     >  ': Space to store REFUEL-SCH is too small')
        CALL LCMGET(IPMAP,'REFUEL-SCH  ',IREFUS)
      ENDIF
      CALL LCMLEN(IPMAP,'REFUEL-CHAN ',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0) THEN 
        IF(ILCMLN .GT. NCHA) CALL XABORT(NAMSBR//
     >  ': Space to store REFUEL-CHAN is too small')
        CALL LCMGET(IPMAP,'REFUEL-CHAN ',REFUT)
      ENDIF
*----
*  Compute number of channels refueled
*----
      DO 100 IC=1,NCHA
        IF(REFUT(IC) .GT. 0.0) NBFUEL=NBFUEL+1
 100  CONTINUE        
      RETURN
      END 
