*DECK LDRCEL
      LOGICAL FUNCTION LDRCEL(IPGEOM,   IT1,    JT1,    IT2,    JT2,
     >                         CELLT, NTYPES,
     >                         IAXIS,   NDIM,   IPRT )
************************************************************************
*                                                                      *
*           NAME: LDRCEL                                               *
*      COMPONENT: DRAGON                                               *
*        VERSION: 1.0                                                  *
*       CREATION: 90/03                                                *
*         AUTHOR: ROBERT ROY                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE WILL CHECK IF TWO CELLS CAN BE CONNECTED*
*                 IN AN ASSEMBLY OF CELLS.                             *
*                                                                      *
*--------+-------------- V A R I A B L E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                 /IO/MOD(DIMENS)*
*--------+----------------------------------------------+--+-----------*
* IPGEOM / POINTER TO THE GEOMETRY (L_GEOM)             /I./INT        *
* IT1    / TYPE OF THE  FIRST CELL.                     /I./INT        *
* JT1    / TURN OF THE  FIRST CELL.                     /I./INT        *
* IT2    / TYPE OF THE SECOND CELL.                     /I./INT        *
* JT2    / TURN OF THE SECOND CELL.                     /I./INT        *
* CELLT  / TO KEEP CELL TYPE NAMES.                     /I./INT(3*NTYP)*
* NTYPES / # OF TYPES.                                  /I./INT        *
* IAXIS  / AXIS OF THE CONNEXION.                       /I./INT        *
* NDIM   / # OF DIMENSIONS (2 OR 3).                    /I./INT        *
* IPRT   / INTERMEDIATE PRINTING LEVEL FOR OUTPUT.      /I./INT        *
*--------+----------------------------------------------+--+-----------*
* LDRCEL / .TRUE.  CELLS:  DO CONNECT ADEQUATELY.                      *
*        / .FALSE.         DO NOT CONNECT ADEQUATELY.                  *
************************************************************************
*
      PARAMETER      ( IOUT=6,NSTATE=20 )
      INTEGER          ISTAT1(NSTATE),ISTAT2(NSTATE),IADD(3),JADD(3),
     >                 CELLT(3*NTYPES),            ILEN(3), JLEN(3)
      CHARACTER        GEOC1*12,GEOC2*12,CAXIS(3)*1,GEOCT1*18,GEOCT2*18
      LOGICAL          LDRGEO
      INTEGER          ID(1)
      COMMON           RD(1)
      EQUIVALENCE     (RD(1),ID(1))
      DATA     CAXIS   / 'X', 'Y', 'Z' /
*
      IF( IAXIS.GT.NDIM )THEN
         LDRCEL= .FALSE.
         CALL XABORT( 'LDRCEL: # OF DIMENSIONS INCOMPATIBLE' )
      ELSE
         LDRCEL= .TRUE.
      ENDIF
      ICX1 = MOD(IAXIS  , NDIM) + 1
      ICX2 = ICX1
      ICY1 = MOD(IAXIS+1, NDIM) + 1
      ICY2 = ICY1
*
*     XY-ROTATES IF NECESSARY FROM JT1 AND JT2
      IF( JT1.EQ.2 )THEN
         IF( ICX1.EQ.2 )THEN
            ICX1= 1
         ELSEIF( ICX1.EQ.1 )THEN
            ICX1= 2
         ENDIF
         IF( ICY1.EQ.2 )THEN
            ICY1= 1
         ELSEIF( ICY1.EQ.1 )THEN
            ICY1= 2
         ENDIF
      ENDIF
      IF( JT2.EQ.2 )THEN
         IF( ICX2.EQ.2 )THEN
            ICX2= 1
         ELSEIF( ICX2.EQ.1 )THEN
            ICX2= 2
         ENDIF
         IF( ICY2.EQ.2 )THEN
            ICY2= 1
         ELSEIF( ICY2.EQ.1 )THEN
            ICY2= 2
         ENDIF
      ENDIF
      N1= 0
      N2= 0
      WRITE( GEOC1(1: 4),'(A4)') CELLT(3*IT1-2)
      WRITE( GEOC1(5: 8),'(A4)') CELLT(3*IT1-1)
      WRITE( GEOC1(9:12),'(A4)') CELLT(3*IT1    )
      IF( IT1.NE.IT2 )THEN
         WRITE( GEOC2(1: 4),'(A4)') CELLT(3*IT2-2 )
         WRITE( GEOC2(5: 8),'(A4)') CELLT(3*IT2-1 )
         WRITE( GEOC2(9:12),'(A4)') CELLT(3*IT2     )
         IF( JT1.EQ.1 )THEN
            GEOCT1= GEOC1//'      '
         ELSE
            GEOCT1= GEOC1//'/TURN '
         ENDIF
         IF( JT2.EQ.1 )THEN
            GEOCT2= GEOC2//'      '
         ELSE
            GEOCT2= GEOC2//'/TURN '
         ENDIF
         IF( IPRT.GT.1 )THEN
            WRITE(IOUT,'(1H ,A1,17H-CELL CONNEXION:  ,2A18)')
     >                    CAXIS(IAXIS),            GEOCT1, GEOCT2
         ENDIF
         IF( .NOT.LDRGEO(IPGEOM, GEOC1, GEOC2, 0) )THEN
*
*           ANALYSE GEOMETRIES
            CALL LCMSIX(IPGEOM, GEOC1, 1)
            CALL XDRSET(ISTAT1,NSTATE,0)
            CALL LCMGET(IPGEOM, 'STATE-VECTOR', ISTAT1)
            IST1= ISTAT1(1)
            IF( IST1.EQ.0 ) THEN
               CALL LCMSIX(IPGEOM, ' ', 2)
               RETURN
            ENDIF
            IF( NDIM.EQ.2 )THEN
               IF( IST1.EQ.5 .OR. IST1.EQ.20 )THEN
                  N1= 1
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICX1), ILEN(1), IT)
                  CALL SETARA( RD, ILEN(1), IADD(1))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICX1), RD(IADD(1)))
               ELSE
                  CALL XABORT( 'LDRCEL: ONE CELL HAS INCORRECT DIM')
               ENDIF
            ELSEIF( NDIM.EQ.3 )THEN
               IF( IST1.EQ.7  .OR. IST1.EQ.21 .OR.
     >             IST1.EQ.22 .OR. IST1.EQ.23 )THEN
                  N1=2
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICX1), ILEN(1), IT)
                  CALL SETARA( RD, ILEN(1), IADD(1))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICX1), RD(IADD(1)))
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICY1), ILEN(2), IT)
                  CALL SETARA( RD, ILEN(2), IADD(2))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICY1), RD(IADD(2)))
                  IF( IAXIS.EQ.IST1-20 )THEN
                     N1=3
                     CALL LCMLEN(IPGEOM, 'RADIUS', ILEN(3), IT)
                     CALL SETARA( RD, ILEN(3), IADD(3))
                     CALL LCMGET(IPGEOM, 'RADIUS', RD(IADD(3)))
                  ENDIF
               ELSE
                  CALL XABORT( 'LDRCEL: ONE CELL HAS INCORRECT DIM')
               ENDIF
            ENDIF
            CALL LCMSIX(IPGEOM, ' ',   2)
*
*           SCAN THE SECOND GEOMETRY
            CALL LCMSIX(IPGEOM, GEOC2, 1)
            CALL XDRSET(ISTAT2,NSTATE,0)
            CALL LCMGET(IPGEOM, 'STATE-VECTOR', ISTAT2)
            IST2= ISTAT2(1)
            IF( IST2.EQ.0 ) THEN
               CALL LCMSIX(IPGEOM, ' ', 2)
               RETURN
            ENDIF
            IF( NDIM.EQ.2 )THEN
               IF( IST2.EQ.5 .OR. IST2.EQ.20 )THEN
                  N2=1
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICX2), JLEN(1), IT)
                  CALL SETARA( RD, JLEN(1), JADD(1))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICX2), RD(JADD(1)))
               ELSE
                  CALL XABORT( 'LDRCEL: ONE CELL HAS INCORRECT DIM')
               ENDIF
            ELSEIF( NDIM.EQ.3 )THEN
               IF( IST2.EQ.7  .OR. IST2.EQ.21 .OR.
     >             IST2.EQ.22 .OR. IST2.EQ.23 )THEN
                  N2=2
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICX2), JLEN(1), IT)
                  CALL SETARA( RD, JLEN(1), JADD(1))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICX2), RD(JADD(1)))
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICY2), JLEN(2), IT)
                  CALL SETARA( RD, JLEN(2), JADD(2))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICY2), RD(JADD(2)))
                  IF( IAXIS.EQ.IST2-20 )THEN
                     N2=3
                     CALL LCMLEN(IPGEOM, 'RADIUS', JLEN(3), IT)
                     CALL SETARA( RD, JLEN(3), JADD(3))
                     CALL LCMGET(IPGEOM, 'RADIUS', RD(JADD(3)))
                  ENDIF
               ELSE
                  CALL XABORT( 'LDRCEL: ONE CELL HAS INCORRECT DIM')
               ENDIF
            ENDIF
            CALL LCMSIX(IPGEOM, ' ',   2)
         ELSE
*
*           GEOMETRIES ARE SIMILAR, CHECK FOR XY-ROTATIONAL INVARIANCE
            IF( JT1.NE.JT2 )THEN
               CALL LCMSIX(IPGEOM, GEOC1, 1)
               CALL XDRSET(ISTAT1,NSTATE,0)
               CALL LCMGET(IPGEOM, 'STATE-VECTOR', ISTAT1)
               IST1= ISTAT1(1)
               IF( IST1.EQ.0 ) THEN
                  CALL LCMSIX(IPGEOM, ' ', 2)
                  RETURN
               ENDIF
               IF( JT1.EQ.1 )THEN
                  GEOCT1= GEOC1//'      '
               ELSE
                  GEOCT1= GEOC1//'/TURN '
               ENDIF
               IF( JT2.EQ.1 )THEN
                  GEOCT2= GEOC1//'      '
               ELSE
                  GEOCT2= GEOC1//'/TURN '
               ENDIF
               IF( IPRT.GT.1 )THEN
                  WRITE(IOUT,'(1H ,A1,17H-CELL CONNEXION:  ,2A18)')
     >                          CAXIS(IAXIS),            GEOCT1, GEOCT2
               ENDIF
               IF( IST1.EQ.5 .OR. IST1.EQ.20 )THEN
                  N1= 1
                  N2= 1
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(1), ILEN(1), IT)
                  CALL SETARA( RD, ILEN(1), IADD(1))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(1), RD(IADD(1)))
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(2), JLEN(1), IT)
                  CALL SETARA( RD, JLEN(1), JADD(1))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(2), RD(JADD(1))) 
               ELSE IF( IST1.EQ.7  .OR. IST1.EQ.21 .OR.
     >             IST1.EQ.22 .OR. IST1.EQ.23 )THEN
                  N1=2
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICX1), ILEN(1), IT)
                  CALL SETARA( RD, ILEN(1), IADD(1))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICX1), RD(IADD(1)))
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICY1), ILEN(2), IT)
                  CALL SETARA( RD, ILEN(2), IADD(2))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICY1), RD(IADD(2)))
                  IF( IAXIS.EQ.IST1-20 )THEN
                     N1=3
                     CALL LCMLEN(IPGEOM, 'RADIUS', ILEN(3), IT)
                     CALL SETARA( RD, ILEN(3), IADD(3))
                     CALL LCMGET(IPGEOM, 'RADIUS', RD(IADD(3)))
                  ENDIF
                  N2=2
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICX2), JLEN(1), IT)
                  CALL SETARA( RD, JLEN(1), JADD(1))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICX2), RD(JADD(1)))
                  CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICY2), JLEN(2), IT)
                  CALL SETARA( RD, JLEN(2), JADD(2))
                  CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICY2), RD(JADD(2)))
                  IF( IAXIS.EQ.IST2-20 )THEN
                     N2=3
                     CALL LCMLEN(IPGEOM, 'RADIUS', JLEN(3), IT)
                     CALL SETARA( RD, JLEN(3), JADD(3))
                     CALL LCMGET(IPGEOM, 'RADIUS', RD(JADD(3)))
                  ENDIF
               ELSE
                  CALL XABORT( 'LDRCEL: ONE CELL HAS INCORRECT DIM')
               ENDIF
               CALL LCMSIX(IPGEOM, ' ',   2)
            ENDIF
         ENDIF
      ELSE
*
*     GEOMETRY ARE OF THE SAME TYPE, CHECK FOR XY-ROTATIONAL INVARIANCE
         IF( JT1.NE.JT2 )THEN
            CALL LCMSIX(IPGEOM, GEOC1, 1)
            CALL XDRSET(ISTAT1,NSTATE,0)
            CALL LCMGET(IPGEOM, 'STATE-VECTOR', ISTAT1)
            IST1= ISTAT1(1)
            IF( IST1.EQ.0 ) THEN
               CALL LCMSIX(IPGEOM, ' ', 2)
               RETURN
            ENDIF
            IF( JT1.EQ.1 )THEN
               GEOCT1= GEOC1//'      '
            ELSE
               GEOCT1= GEOC1//'/TURN '
            ENDIF
            IF( JT2.EQ.1 )THEN
               GEOCT2= GEOC1//'      '
            ELSE
               GEOCT2= GEOC1//'/TURN '
            ENDIF
            IF( IPRT.GT.1 )THEN
               WRITE(IOUT,'(1H ,A1,17H-CELL CONNEXION:  ,2A18)')
     >                       CAXIS(IAXIS),            GEOCT1, GEOCT2
            ENDIF
            IF( IST1.EQ.5 .OR. IST1.EQ.20 )THEN
               N1= 1
               N2= 1
               CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(1), ILEN(1), IT)
               CALL SETARA( RD, ILEN(1), IADD(1))
               CALL LCMGET(IPGEOM, 'MESH'//CAXIS(1), RD(IADD(1)))
               CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(2), JLEN(1), IT)
               CALL SETARA( RD, JLEN(1), JADD(1))
               CALL LCMGET(IPGEOM, 'MESH'//CAXIS(2), RD(JADD(1)))
            ELSE IF( IST1.EQ.7  .OR. IST1.EQ.21 .OR.
     >          IST1.EQ.22 .OR. IST1.EQ.23 )THEN
               N1=2
               CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICX1), ILEN(1), IT)
               CALL SETARA( RD, ILEN(1), IADD(1))
               CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICX1), RD(IADD(1)))
               CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICY1), ILEN(2), IT)
               CALL SETARA( RD, ILEN(2), IADD(2))
               CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICY1), RD(IADD(2)))
               IF( IAXIS.EQ.IST1-20 )THEN
                  N1=3
                  CALL LCMLEN(IPGEOM, 'RADIUS', ILEN(3), IT)
                  CALL SETARA( RD, ILEN(3), IADD(3))
                  CALL LCMGET(IPGEOM, 'RADIUS', RD(IADD(3)))
               ENDIF
               N2=2
               CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICX2), JLEN(1), IT)
               CALL SETARA( RD, JLEN(1), JADD(1))
               CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICX2), RD(JADD(1)))
               CALL LCMLEN(IPGEOM, 'MESH'//CAXIS(ICY2), JLEN(2), IT)
               CALL SETARA( RD, JLEN(2), JADD(2))
               CALL LCMGET(IPGEOM, 'MESH'//CAXIS(ICY2), RD(JADD(2)))
               IF( IAXIS.EQ.IST2-20 )THEN
                  N2=3
                  CALL LCMLEN(IPGEOM, 'RADIUS', JLEN(3), IT)
                  CALL SETARA( RD, JLEN(3), JADD(3))
                  CALL LCMGET(IPGEOM, 'RADIUS', RD(JADD(3)))
               ENDIF
            ELSE
               CALL XABORT( 'LDRCEL: ONE CELL HAS INCORRECT DIM')
            ENDIF
            CALL LCMSIX(IPGEOM, ' ',   2)
         ENDIF
      ENDIF
*
*     CHECK VECTORS THAT HAVE BEEN EXTRACTED AND RELEASE SPACE...
      IF( N1.EQ.N2 )THEN
         DO 20 I= 1, N1
            IF( ILEN(I).NE.JLEN(I) )
     >         CALL XABORT( 'LDRCEL: INCORRECTED MESHING DIM')
            DO 10 J= 1, ILEN(I)
            IF( RD(J-1+IADD(I)).NE.RD(J-1+JADD(I)))
     >         CALL XABORT( 'LDRCEL: INCORRECTED MESHING')
   10       CONTINUE
            CALL RLSARA( RD(JADD(I)) )
            CALL RLSARA( RD(IADD(I)) )
   20    CONTINUE
      ELSE
         CALL XABORT( 'LDRCEL: TYPES ARE INCOMPATIBLE')
      ENDIF
*     RETURN
      END
