*DECK LIBMAC
      SUBROUTINE LIBMAC(IPLIB ,IPLIBX,IPRINT,MAXISO,NBISO ,NBISOX,
     >                  NBMIX ,NBMIXX,NGRO  ,TMPDAY,
     >                  ISONAM,ISONMX,MIX   ,DEN   ,DENMIX,MIXIX ,
     >                  DENIX ,LOCUPD,LISM  ,DENMOD,MASKM,MASKG  )
C
C-----------------------------------------------------------------------
C
C READ THE INFORMATION RELATED TO MICROSCOPIC CROSS SECTION LIBRARY.
C
C INPUT/OUTPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C            (L_LIBRARY SIGNATURE).
C  IPLIBX  : POINTER TO THE LATTICE NEW LIBRARY DENSITY.
C  IPRINT  : PRINT FLAG.
C  MAXISO  : MAXIMUM NUMBER OF ISOTOPES PERMITTED
C  NBISO   : NUMBER OF ISOTOPES PRESENT ON IPLIB
C  NBISOX  : NUMBER OF ISOTOPES PRESENT ON IPLIBX IF MAXS
C            BURNUP STEP ON IPLIBX IF BURN
C  NBMIX   : NUMBER OF MIXTURES DEFINED ON IPLIB
C  NBMIXX  : NUMBER OF MIXTURES DEFINED ON IPLIBX
C  NGRO    : NUMBER OF ENERGY GROUPS.
C  TMPDAY  : TIME/BURNUP/IRRADIATION STAMP IN DAYS TMPDAY(3)
C  ISONAM  : OLD NAME OF ISOTOPES. ISONAM(NTC,MAXISO)
C  ISONMX  : NEW NAME OF ISOTOPES. ISONMX(NTC,MAXISO)
C  MIX     : MIX NUMBER OF EACH ISOTOPE ON IPLIB (CAN BE ZERO)
C            INTEGER MIX(MAXISO)
C  DEN     : DENSITY OF EACH ISOTOPE ON IPLIB
C            REAL DEN(MAXISO)
C  DENMIX  : DENSITY OF MIXTURE ON IPLIB (CAN BE -1.0)
C            REAL DENMIX(NBMIX)
C  MIXIX   : MIX NUMBER OF EACH ISOTOPE ON IPLIBX (CAN BE ZERO)
C            INTEGER MIXIX(MAXISO)
C  DENIX   : DENSITY OF EACH ISOTOPE ON IPLIBX
C            REAL DENIX(MAXISO)
C
C SCRATCH STORAGE ALLOCATED IN THE CALLING PROGRAM:
C  LOCUPD  : LOCATION OF IPLIB MIXTURE IN IPLIBX
C            INTEGER LOCUPD(NBMIX)
C  LISM    : LOCATION IN IPLIB OF ISOTOPE ASSOCIATED WITH A MIXTURE
C            INTEGER LISM(MAXISO)
C  DENMOD  : MODIFIED DENSITY OF EACH ISOTOPE ON IPLIB
C            REAL DENMOD(NBISO)
C  MASKM   : MIXTURE UPDATE MASK
C            LOGICAL MASKM(NBMIX)
C  MASKG   : GROUP UPDATE MASK
C            LOGICAL MASKG(NGRO)
C
C--------------------------------------- AUTHOR: A. HEBERT 12/04/88 ----
C
      IMPLICIT     NONE
      INTEGER      IOUT,NTC
      CHARACTER    NAMSBR*6
      PARAMETER   (IOUT=6,NTC=3,NAMSBR='LIBMAC')
      INTEGER      NSTATE
      PARAMETER   (NSTATE=20)
C----
C  ROUTINE PARAMETERS
C----
      INTEGER      IPLIB,IPLIBX,IPRINT,MAXISO,NBISO,NBISOX,
     >             NBMIX,NBMIXX,NGRO
      INTEGER      ISONAM(NTC,MAXISO),ISONMX(NTC,MAXISO),
     >             MIX(MAXISO),MIXIX(MAXISO),LOCUPD(NBMIX),
     >             LISM(MAXISO)
      REAL         TMPDAY(3),DEN(MAXISO),DENMIX(NBMIX),
     >             DENIX(MAXISO),DENMOD(MAXISO)
      LOGICAL      MASKM(NBMIX),MASKG(NGRO)
C----
C  MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE
      COMMON       RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      NISOB,IMIXB,IDENB
C----
C  INPUT
C----
      INTEGER      ITYPLU,INTLIR
      CHARACTER    CARLIR*12
      REAL         REALIR
      DOUBLE PRECISION DBLLIR
C----
C  LOCAL PARAMETERS
C----
      INTEGER      KCHAR(NTC),IGR,ISO,JSO,IMIX,NISOM,ITSTMP,NNMIX,
     >             MODISO,NMIXUP,NIUPD,IMIXX,ILCMLN,ILCMTY,ITC
      INTEGER      ISTATE(NSTATE),MXISOB
C----
C  READ ORIGINAL ISOTOPE AND MIXTURE INFORMATION FROM IPLIB
C----
      CALL LCMGET(IPLIB,'ISOTOPESDENS',DEN)
      CALL LCMGET(IPLIB,'ISOTOPESMIX',MIX)
      CALL LCMGET(IPLIB,'ISOTOPESUSED',ISONAM)
      DO 10 JSO=1,NBISO
        DENMOD(JSO)=DEN(JSO)
 10   CONTINUE
      DO 20 IMIX=1,NBMIX
        DENMIX(IMIX)=1.0
        LOCUPD(IMIX)=0
 20   CONTINUE
C----
C  WRITE ORIGINAL MATERIAL COMPOSITION IF REQUIRED
C----
      IF(IPRINT.GT.0) THEN
        WRITE(IOUT,6000) NGRO,NBISO,NBMIX
        DO 600 IMIX=1,NBMIX
          NISOM=0
          DO 601 ISO=1,NBISO
            IF(MIX(ISO).EQ.IMIX) THEN
               NISOM=NISOM+1
               LISM(NISOM)=ISO
            ENDIF
 601      CONTINUE
          WRITE(IOUT,6010) IMIX
          WRITE(IOUT,6011) ((ISONAM(ITC,LISM(ISO)),ITC=1,NTC-1),
     >      DEN(LISM(ISO)),ISO=1,NISOM)
 600    CONTINUE
      ENDIF
C----
C  READ ISOTOPE AND MIXTURE INFORMATION FROM IPLIBX
C----
      ITSTMP=2
      IF(NBISOX.GT.0) THEN
        CALL LCMGET(IPLIBX,'ISOTOPESDENS',DENIX)
        CALL LCMGET(IPLIBX,'ISOTOPESMIX',MIXIX)
        CALL LCMGET(IPLIBX,'ISOTOPESUSED',ISONMX)
        IF(IPRINT.GT.0) THEN
          WRITE(IOUT,6001) NBISOX,NBMIXX
          DO 610 IMIX=1,NBMIXX
            NISOM=0
            DO 611 ISO=1,NBISOX
              IF(MIXIX(ISO).EQ.IMIX) THEN
                 NISOM=NISOM+1
                 LISM(NISOM)=ISO
              ENDIF
 611        CONTINUE
            WRITE(IOUT,6010) IMIX
            WRITE(IOUT,6011) ((ISONMX(ITC,LISM(ISO)),ITC=1,NTC-1),
     >        DENIX(LISM(ISO)),ISO=1,NISOM)
 610      CONTINUE
        ENDIF
      ELSE IF(NBISOX.LT.0) THEN
        WRITE(IOUT,6000)
        CALL LCMGET(IPLIBX,'STATE-VECTOR',ISTATE)
        MXISOB=ISTATE(4)
        CALL SETARA(IBASE,MXISOB,IMIXB)        
        CALL SETARA(IBASE,3*MXISOB,NISOB)        
        CALL SETARA(RBASE,MXISOB,IDENB)
        CALL LCMGET(IPLIBX,'ISONAMES-EVO',IBASE(NISOB))
        CALL LCMGET(IPLIBX,'ISOMIXT-EVO',IBASE(IMIXB))
        WRITE(CARLIR,'(8HDEPL-DAT,I4)') -NBISOX
        CALL LCMSIX(IPLIBX,CARLIR,1)
        CALL LCMGET(IPLIBX,'ISOTOPESDENS',RBASE(IDENB))
*        CALL LCMGET(IPLIBX,'ISOTOPESDENS',DENMOD)
*----
*  Identify isotopes on burnup with isotopes on library
*  and transfer density 
*----
        DO 200 IMIX=1,NBMIX
          DO 201 ISO=1,NBISO
            IF(MIX(ISO).EQ.IMIX) THEN
              DO 210 JSO=0,MXISOB-1
                IF(IBASE(IMIXB+JSO)     .EQ. IMIX          .AND.
     >             IBASE(NISOB+3*JSO)   .EQ. ISONAM(1,ISO) .AND.
     >             IBASE(NISOB+3*JSO+1) .EQ. ISONAM(2,ISO)) THEN
                  IF(IPRINT .GE. 10) THEN
                    WRITE(IOUT,6020) IMIX,ISO,
     >             (ISONAM(ITC,ISO),ITC=1,NTC-1),DENMOD(ISO),
     >              JSO+1,IBASE(NISOB+3*JSO),IBASE(NISOB+3*JSO+1),
     >              RBASE(IDENB+JSO)
                  ENDIF
                  DENMOD(ISO)=RBASE(IDENB+JSO)
                  GO TO  215
                ENDIF
 210          CONTINUE
            ENDIF
 215        CONTINUE
 201      CONTINUE
 200    CONTINUE
        CALL RLSARA(RBASE(IDENB))
        CALL RLSARA(IBASE(IMIXB))
        CALL RLSARA(IBASE(NISOB))
        CALL LCMGET(IPLIBX,'BURNUP-IRRAD',TMPDAY(2))
        CALL LCMSIX(IPLIBX,CARLIR,2)
        IF(IPRINT.GT.0) THEN
          DO 620 IMIX=1,NBMIX
            NISOM=0
            DO 621 ISO=1,NBISO
              IF(MIX(ISO).EQ.IMIX) THEN
                 NISOM=NISOM+1
                 LISM(NISOM)=ISO
              ENDIF
 621        CONTINUE
            WRITE(IOUT,6010) IMIX
            WRITE(IOUT,6011) ((ISONAM(ITC,LISM(ISO)),ITC=1,NTC-1),
     >        DENMOD(LISM(ISO)),ISO=1,NISOM)
 620      CONTINUE
        ENDIF
      ENDIF
C----
C  READ UPDATE INFORMATION FROM INPUT
C  FORMAT PERMITTED ARE
C  [MIX IMLIB [ IMLIBX ] [ DENMOD ] [ NAMISO CONCM(I) ] ] ;
C  DEFAULT:
C    MIX ABSENT, IPLIBX >0     -> ALL ISOTOPES AND ALL MIXTURES
C    MIX ABSENT, IPLIBX =0     -> NO UPDATE - PRINT ONLY
C    IMLIBX ABSENT, IPLIBX >0  -> CONCF(ISO,IMX)=CONC(ISO,IMLIBX)
C    IMLIBX ABSENT, IPLIBX =0  -> CONCF(ISO,IMX)=CONC(ISO,-IMLIB)
C    IMLIBX > 0                -> CORRECTION FROM IPLIBX
C    IMLIBX = -IMLIB           -> CORRECTION FROM IPLIB
C    DENMOD PRESENT            -> CONCF(I,IMX)=CONCF(I,IMX)*DENMOD
C    NAMISO ABSENT, IPLIBX >0  -> ALL ISOTOPE FOR MIXTURE
C    NAMISO ABSENT, IPLIBX =0  -> ALL ISOTOPE FOR MIXTURE
C    NAMISO PRESENT, CONCM >=0 -> ISOTOPE SPECIFIED
C                                 CONCF(I,IMX)=DENMOD*CONCM(I,IMX)
C    NAMISO PRESENT, CONCM <0  -> ISOTOPE SPECIFIED
C                                 CONCF(I,IMX)=CONC(ISO,-IMLIB)*DENMOD
C----
      NNMIX=0
 100  CONTINUE
      CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
 101  CONTINUE
      IF(ITYPLU .NE. 3) CALL XABORT(NAMSBR//
     >': CHARACTER DATA EXPECTED.')
      IF(CARLIR .EQ. ';') THEN
        GO TO 105
      ELSE IF(CARLIR(1:3) .EQ. 'MIX') THEN
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 1) CALL XABORT(NAMSBR//
     >  ': MIXTURE TO UPDATE MUST BE GIVEN.')
        NNMIX=INTLIR
        IF(NNMIX .GT. NBMIX) THEN
          CALL XABORT(NAMSBR//': CANNOT UPDATE THIS MIXTURE.')
        ELSE IF(NNMIX .LE. 0) THEN
          CALL XABORT(NAMSBR//': MIX NUMBER .LE. 0.')
        ENDIF
        IF(NBISOX .GT. 0) THEN
          LOCUPD(NNMIX)=NNMIX
        ELSE
          LOCUPD(NNMIX)=-NNMIX
        ENDIF
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(NBISOX .GT. 0) THEN
          IF(ITYPLU .EQ. 1) THEN
            IF(INTLIR .LE. 0 .OR.
     >         INTLIR .GT. NBMIXX) CALL XABORT(NAMSBR//
     >         ': CANNOT UPDATE THIS MIXTURE.')
            LOCUPD(NNMIX)=INTLIR
            CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
          ENDIF
        ENDIF
        IF(ITYPLU .EQ. 2) THEN
          DENMIX(NNMIX)=REALIR
          CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        ENDIF
        GO TO 101
      ENDIF
      READ(CARLIR,'(2A4)') (KCHAR(ITC),ITC=1,NTC-1)
      IF(LOCUPD(NNMIX) .LT. 0) THEN
        DO 40 ISO=1,NBISO
          IF(KCHAR(1) .EQ. ISONAM(1,ISO) .AND.
     >       KCHAR(2) .EQ. ISONAM(2,ISO) .AND.
     >       NNMIX .EQ. MIX(ISO)) THEN
             MODISO=ISO
             GO TO 45
          ENDIF
 40     CONTINUE
        WRITE(IOUT,'(10H MIXTURE :,1X,I10,10H ISOTOPE :,1X,2A4)')
     >    NNMIX,(KCHAR(ITC),ITC=1,NTC-1)
        CALL XABORT(NAMSBR//
     >  ': CANNOT UPDATE THIS ISOTOPE IN CURRENT MIXTURE.')
 45     CONTINUE
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': NEW ISOTOPIC DENSITY EXPECTED.')
        DENMOD(MODISO)=REALIR
      ELSE IF(LOCUPD(NNMIX) .GT. 0) THEN
        DO 50 ISO=1,NBISOX
          IF(KCHAR(1) .EQ. ISONMX(1,ISO) .AND.
     >       KCHAR(2) .EQ. ISONMX(2,ISO) .AND.
     >       LOCUPD(NNMIX) .EQ. MIXIX(ISO)) THEN
           MODISO=ISO
           GO TO 55
        ENDIF
 50     CONTINUE
        WRITE(IOUT,'(10H MIXTURE :,1X,I10,10H ISOTOPE :,1X,2A4)')
     >    NNMIX,(KCHAR(ITC),ITC=1,NTC-1)
        CALL XABORT(NAMSBR//
     >  ': CANNOT UPDATE THIS ISOTOPE IN CURRENT MIXTURE.')
 55     CONTINUE
        CALL REDGET(ITYPLU,INTLIR,REALIR,CARLIR,DBLLIR)
        IF(ITYPLU .NE. 2) CALL XABORT(NAMSBR//
     >  ': NEW ISOTOPIC DENSITY EXPECTED.')
        IF(REALIR .LT. 0.0) THEN
          DENIX(MODISO)=-DENIX(MODISO)
        ELSE
          DENIX(MODISO)=REALIR
        ENDIF
      ENDIF
      GO TO 100
 105  CONTINUE
      IF(NNMIX .EQ. 0) THEN
        IF(NBISOX .GT. 0) THEN
          DO 60 IMIX=1,NBMIX
            LOCUPD(IMIX)=IMIX
 60       CONTINUE
        ELSE IF(NBISOX .LT. 0) THEN
          DO 61 IMIX=1,NBMIX
            LOCUPD(IMIX)=-IMIX
 61       CONTINUE
        ENDIF
      ENDIF
C----
C  TRANSFER DENSITY FROM DENMOD OR DENIX TO DEN IF REQUIRED
C----
      NMIXUP=0
      DO 70 IMIX=1,NBMIX
        MASKM(IMIX)=.FALSE.
        IF(LOCUPD(IMIX) .GT. 0) THEN
          NIUPD=0
          IMIXX=LOCUPD(IMIX)
          DO 71 ISO=1,NBISOX
            IF(MIXIX(ISO) .EQ. IMIXX) THEN
              DO 72 JSO=1,NBISO
                IF(ISONAM(1,JSO) .EQ. ISONMX(1,ISO) .AND.
     >             ISONAM(2,JSO) .EQ. ISONMX(2,ISO) .AND.
     >             MIX(JSO)      .EQ. IMIX) THEN
                  IF(DENMIX(IMIX)*DENIX(ISO) .NE. DEN(JSO)) THEN
                    DEN(JSO)=DENMIX(IMIX)*DENIX(ISO)
                    NIUPD=NIUPD+1
                  ENDIF
                ENDIF
 72           CONTINUE
            ENDIF
 71       CONTINUE
          IF(NIUPD .NE. 0) THEN
            MASKM(IMIX)=.TRUE.
            NMIXUP=NMIXUP+1
          ENDIF
        ELSE IF(LOCUPD(IMIX) .LT. 0) THEN
          NIUPD=0
          DO 73 JSO=1,NBISO
            IF(MIX(JSO) .EQ. IMIX) THEN
              IF(DENMIX(IMIX)*DENMOD(JSO) .NE. DEN(JSO)) THEN
                IF(DENMIX(IMIX)*DENMOD(JSO) .GE. 0.0) THEN
                  DEN(JSO)=DENMIX(IMIX)*DENMOD(JSO)
                  NIUPD=NIUPD+1
                ENDIF
              ENDIF
            ENDIF
 73       CONTINUE
          IF(NIUPD .NE. 0) THEN
            MASKM(IMIX)=.TRUE.
            NMIXUP=NMIXUP+1
          ENDIF
        ENDIF
 70   CONTINUE
C----
C  UPDATE ALL MATERIAL IF MACROLIB DIRECTORY ABSENT
C----
      CALL LCMLEN(IPLIB,'MACROLIB',ILCMLN,ILCMTY)
      IF(ILCMLN .EQ. 0) THEN
        NMIXUP=NBMIX
        DO 75 IMIX=1,NBMIX
          MASKM(IMIX)=.TRUE.
 75     CONTINUE
        IF(IPRINT .GT. 0) WRITE(IOUT,6004)
      ENDIF
C----
C  WRITE UPDATED MATERIAL COMPOSITION IF REQUIRED
C----
      IF(NMIXUP .GT. 0) THEN
        IF(IPRINT .GT. 0) THEN
          WRITE(IOUT,6002)
          DO 630 IMIX=1,NBMIX
            NISOM=0
            IF(MASKM(IMIX)) THEN
              DO 631 ISO=1,NBISO
                IF(MIX(ISO) .EQ. IMIX) THEN
                   NISOM=NISOM+1
                   LISM(NISOM)=ISO
                ENDIF
 631          CONTINUE
              WRITE(IOUT,6010) IMIX
              WRITE(IOUT,6011) ((ISONAM(ITC,LISM(ISO)),ITC=1,NTC-1),
     >          DEN(LISM(ISO)),ISO=1,NISOM)
            ENDIF
 630      CONTINUE
        ENDIF
C----
C  SAVE ISOTOPE NEW DENSITY
C----
        CALL LCMPUT(IPLIB,'ISOTOPESDENS',NBISO,2,DEN)
C----
C COMPUTE THE MACROSCOPIC X-SECTIONS
C----
        DO 80 IGR=1,NGRO
          MASKG(IGR)=.TRUE.
 80     CONTINUE
        CALL LIBMIX(IPLIB ,NGRO  ,NBISO ,ISONAM,MIX   ,DEN   ,
     >              MASKM ,MASKG ,ITSTMP,TMPDAY)
      ELSE
        IF(IPRINT .GT. 0) WRITE(IOUT,6003)
      ENDIF
      RETURN
C----
C  FORMAT
C----
 6000 FORMAT(/' LIBMAC: MODIFIED LIBRARY PROPERTIES '/
     >        '         NUMBER OF GROUPS   = ',I10/
     >        '         NUMBER OF ISOTOPES = ',I10/
     >        '         NUMBER OF MIXTURES = ',I10/
     >        '         ORIGINAL NUMBER DENSITIES IN MIXTURES',
     >        ' FOLLOWS')
 6001 FORMAT(/' LIBMAC: DENSITY LIBRARY PROPERTIES '/
     >        '         NUMBER OF ISOTOPES = ',I10/
     >        '         NUMBER OF MIXTURES = ',I10/
     >        '         ORIGINAL NUMBER DENSITIES IN MIXTURES',
     >        '  FOLLOWS')
 6002 FORMAT(/' LIBMAC: FINAL NUMBER DENSITIES MIXTURES FOLLOWS')
 6003 FORMAT(/' LIBMAC: NO UPDATED MIXTURES')
 6004 FORMAT(/' LIBMAC: MACROSCOPIC ABSENT -> ALL MIXTURES UPDATED')
 6010 FORMAT(/' ISOTOPIC DENSITIES FOR MIXTURE = ',I3)
 6011 FORMAT(1P,5(4X,2A4,':',E12.4))
 6020 FORMAT(1X,' REPLACING MIXTURE=',I5,
     >       1X,'ISOTOPE=',I5,
     >       1X,'NAME =',2A4,
     >       1X,'DENSITY =',1P,E15.7,0P,
     >       1X,'WITH BURNUP ISOTOPE=',I5,
     >       1X,'NAME =',2A4,
     >       1X,'DENSITY =',1P,E15.7)
      END
