*DECK LIBMIX
      SUBROUTINE LIBMIX(IPLIB,NGROUP,NBISO,ISONAM,MIX,DEN,MASK,MASKL,
     >                  ITSTMP,TMPDAY)
C
C-----------------------------------------------------------------------
C
C TRANSFORMATION OF THE ISOTOPE ORDERED MICROSCOPIC CROSS SECTIONS TO
C GROUP ORDERED MACROSCOPIC CROSS SECTIONS (PART 1).
C
C INPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C            (L_LIBRARY SIGNATURE).
C  NGROUP  : NUMBER OF ENERGY GROUPS.
C  NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  ISONAM  : NAME OF EACH ISOTOPE. ISONAM(3,NBISO)
C  MIX     : MIXTURE NUMBER OF EACH ISOTOPE. CAN BE ZERO.
C            DIMENSION MIX(NBISO)
C  DEN     : DENSITY OF EACH ISOTOPE. DIMENSION DEN(NBISO)
C  MASK    : =.TRUE. IF A MIXTURE IS TO BE MADE.
C            DIMENSION MASK(NBMIX) WHERE NBMIX=MAX(MIX(I))
C  MASKL   : =.TRUE. IF AN ENERGY GROUP IS TO BE TREATED.
C            DIMENSION MASKL(NGROUP)
C  ITSTMP  : = 0 NO PERTURBATIONS   / OLD TIME STAMP
C            = 1 NO PERTURBATIONS   / NEW TIME STAMP
C            = 2 WITH PERTURBATIONS / NEW TIME STAMP
C  TMPDAY  : TIME IN DAYS/BURNUP/IRRADIATION STAMP
C
C--------------------------------------- AUTHOR: A. HEBERT 19/04/88 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER      NSTATE,NDPROC,MAXED
      PARAMETER   (NSTATE=20,NDPROC=20,MAXED=50)
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      LOGICAL      LBASE(1)
      REAL         RBASE(1)
      COMMON       RBASE
      EQUIVALENCE (RBASE(1),IBASE(1),LBASE(1))
      INTEGER      IJCEDM,LMASKG,IFGUPS,IPXS,IPSC,IADDXS,ISONFI,
     1             INOFIS,ISCAT,IIPOS,IIJJ,INJJ,ISCT,IGXS,IGAS,ISOSCT,
     2             INIFIS,INDPRO,ITYPRO,IXSREC,IEN,IDU
C----
C INTERFACE VARIABLES
C----
      INTEGER      IPLIB,NGROUP,NBISO
      INTEGER      ISONAM(3,NBISO),MIX(NBISO),ITSTMP
      REAL         DEN(NBISO),TMPDAY(3),OLDTIM(3)
      LOGICAL      MASK(*),MASKL(NGROUP)
C----
C LOCAL VARIABLES
C----
      INTEGER      NL,ITRANC,NED,IDEB,J,NBMIX,I,LENGTH,ITYLCM,IGR,
     1             NUPD,MXNL,MXED,NOFI,MXFI,MXXS,LLFG,ITFG,JPSC,IED,
     2             JED,IOFPXS,NNFI,MAXGAS,NPROC
      INTEGER      ISTATE(NSTATE),KCHAR(3),IVECT(MAXED)
      CHARACTER    TEXT12*12,HVECT(MAXED)*6,
     1             HSMG*131
      LOGICAL      MASKK
C----
C   RECOVER SOME LIBRARY PARAMETERS.
C----
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      NL=ISTATE(4)
      ITRANC=ISTATE(5)
      NED=ISTATE(13)
      CALL SETARA(IBASE,2*NED,IJCEDM)
      CALL LCMGET(IPLIB,'HVECT',IBASE(IJCEDM))
      IDEB=IJCEDM
      DO 10 J=1,NED
        WRITE(HVECT(J),'(A4,A2)') IBASE(IDEB),IBASE(IDEB+1)
        IDEB=IDEB+2
   10 CONTINUE
      CALL RLSARA(IBASE(IJCEDM))
C----
C  BUILD THE MACROSCOPIC CROSS SECTIONS.
C----
      NBMIX=0
      DO 20 I=1,NBISO
   20 NBMIX=MAX(NBMIX,MIX(I))
C----
C  LOOK FOR OLD LIBRARY DATA
C----
      CALL SETARA(LBASE,NGROUP,LMASKG)
      CALL LCMLEN(IPLIB,'MACROLIB',LENGTH,ITYLCM)
      MASKK=(LENGTH.EQ.-1)
      IF(MASKK) THEN
        CALL LCMSIX(IPLIB,'MACROLIB',1)
        CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
        WRITE(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
        IF(TEXT12.NE.'L_MACROLIB') THEN
          CALL XABORT('LIBMIX: INVALID SIGNATURE ON THE MACROLIB.')
        ENDIF
        CALL LCMLEN(IPLIB,'TIMESTAMP',LENGTH,ITYLCM)
        IF(LENGTH.GT.0.AND.LENGTH.LE.3) THEN
          CALL LCMGET(IPLIB,'TIMESTAMP',OLDTIM)
          IF(ITSTMP.EQ.0) THEN
            TMPDAY(1)=OLDTIM(1)
            TMPDAY(2)=OLDTIM(2)
            TMPDAY(3)=OLDTIM(3)
          ENDIF
        ENDIF
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        IF((ISTATE(1).NE.NGROUP).OR.(ISTATE(6)*ITRANC.LT.0)) THEN
           WRITE(HSMG,9000) ISTATE(1),ISTATE(6),NGROUP,ITRANC
           CALL XABORT(HSMG)
        ELSE IF(ISTATE(2).GT.NBMIX) THEN
           WRITE(HSMG,9001) ISTATE(2),NBMIX
           CALL XABORT(HSMG)
        ENDIF
        IF(NBMIX.GT.ISTATE(2)) THEN
          DO 140 IGR=1,NGROUP
            LBASE(LMASKG+IGR-1)=.TRUE.
 140      CONTINUE
          NUPD=NGROUP
        ELSE
          NUPD=0
          DO 141 IGR=1,NGROUP
            LBASE(LMASKG+IGR-1)=MASKL(IGR)
            IF(MASKL(IGR)) NUPD=NUPD+1
 141      CONTINUE
        ENDIF
C----
C  ALLOCATE MEMORY FOR IPXS
C----
        MXNL=MAX(NL,ISTATE(3))
        MXED=ISTATE(5)+NED
        NOFI=ISTATE(4)
        MXFI=NBISO+NOFI
        MXXS=2+MXED+3*MXFI
        CALL SETARA(IBASE,MXNL*NBMIX,IFGUPS)
        CALL XDRSET(IBASE(IFGUPS),MXNL*NBMIX,NGROUP+1)
        CALL LCMLEN(IPLIB,'FGWITHUPSCAT',LLFG,ITFG)
        IF(LLFG.EQ.0) THEN
          CALL XDRSET(IBASE(IFGUPS),MXNL*NBMIX,NGROUP+1)
        ELSE
          CALL LCMGET(IPLIB,'FGWITHUPSCAT',IBASE(IFGUPS))
        ENDIF
        CALL SETARA(IBASE,MXXS,IPXS)
        CALL SETARA(IBASE,MXNL,IPSC)
        CALL XDRSET(IBASE(IPXS),MXXS,0)
        CALL XDRSET(IBASE(IPSC),MXNL,0)
C----
C  SET MAXIMUM SCATTERING LEVEL
C----
        DO 100 JPSC=IPSC,IPSC+ISTATE(3)-1
          IBASE(JPSC)=1
 100    CONTINUE
C----
C  SET TOTAL
C----
        IBASE(IPXS)=1
        IF(ITRANC.GT.0) IBASE(IPXS+1)=1
C----
C  SET ADDITIONAL XS.
C  1) SET XS ALREADY PRESENT
C  2) SET NEW XS
C----
        CALL LCMLEN(IPLIB,'ADDXSNAME',LENGTH,ITYLCM)
        IF(LENGTH.GT.0) THEN
          IDEB=IPXS+1+ISTATE(5)
          DO 110 IED=NED,1,-1
            HVECT(ISTATE(5)+IED)=HVECT(IED)
            IBASE(IDEB+IED)=0
 110      CONTINUE
          CALL SETARA(IBASE,2*MXED,IADDXS)
          CALL LCMGET(IPLIB,'ADDXSNAME',IBASE(IADDXS))
          IDEB=IPXS+1
          DO 120 JED=1,ISTATE(5)
            IBASE(IDEB+JED)=1
            WRITE(HVECT(JED),'(A4,A2)')
     >       (IBASE(IADDXS+2*(JED-1)+I),I=0,1)
            DO 121 IED=1,NED
              IF(HVECT(JED).EQ.HVECT(ISTATE(5)+IED)) THEN
                HVECT(ISTATE(5)+IED)='      '
                IBASE(IDEB+ISTATE(5)+IED)=0
                GO TO 125
              ENDIF
 121        CONTINUE
 125        CONTINUE
 120      CONTINUE
          CALL RLSARA(IBASE(IADDXS))
        ENDIF
        CALL LCMSIX(IPLIB,' ',2)
      ELSE
C----
C  Transfer ENERGY and DELTAU to MACROLIB
C----
        CALL SETARA(RBASE,NGROUP+1,IEN)
        CALL SETARA(RBASE,NGROUP,IDU)
        CALL LCMGET(IPLIB,'ENERGY',RBASE(IEN))
        CALL LCMGET(IPLIB,'DELTAU',RBASE(IDU))
        CALL LCMSIX(IPLIB,'MACROLIB',1)  
        CALL LCMPUT(IPLIB,'ENERGY',NGROUP+1,2,RBASE(IEN))
        CALL LCMPUT(IPLIB,'DELTAU',NGROUP,2,RBASE(IDU))
        CALL LCMSIX(IPLIB,'MACROLIB',2) 
        CALL RLSARA(RBASE(IDU)) 
        CALL RLSARA(RBASE(IEN)) 
        CALL XDRSET(ISTATE,NSTATE,0)
        ISTATE(1)=NGROUP
        ISTATE(2)=NBMIX
        ISTATE(3)=NL
        ISTATE(4)=0
        ISTATE(5)=0
        ISTATE(6)=ITRANC
        NOFI=0
        MXNL=NL
        MXED=NED
        MXFI=NBISO
        MXXS=2+MXED+3*MXFI
        CALL SETARA(IBASE,MXNL*NBMIX,IFGUPS)
        CALL XDRSET(IBASE(IFGUPS),MXNL*NBMIX,NGROUP+1)
        CALL SETARA(IBASE,MXXS,IPXS)
        CALL SETARA(IBASE,MXNL,IPSC)
        CALL XDRSET(IBASE(IPXS),MXXS,0)
        CALL XDRSET(IBASE(IPSC),MXNL,0)
        NUPD=0
        DO 142 IGR=1,NGROUP
          LBASE(LMASKG+IGR-1)=MASKL(IGR)
          IF(MASKL(IGR)) NUPD=NUPD+1
 142    CONTINUE
      ENDIF
C----
C  UPDATE MACROSCOPIC XS IF REQUIRED
C----
      IF(NUPD.GT.0) THEN
        CALL SETARA(IBASE,2*MXFI,ISONFI)
        IOFPXS=IPXS+MXED+2
        CALL SETARA(IBASE,NBMIX*MXFI,INIFIS)
        CALL XDRSET(IBASE(INIFIS),NBMIX*MXFI,0)
        CALL SETARA(IBASE,NBISO,INOFIS)
C----
C  CHECK FOR FISSILE ISOTOPES
C----
        CALL LIBFIS(IPLIB,NBISO,NBMIX,NOFI,NNFI,ISONAM,
     >              IBASE(ISONFI),MIX,IBASE(IOFPXS),IBASE(INIFIS),
     >              IBASE(INOFIS))
C----
C  SCAN FOR ADDITIONAL AND STANDARD CROSS SECTIONS TO BE SAVED
C----
        NPROC=NDPROC+NL
        CALL SETARA(IBASE,NPROC,INDPRO)
        CALL XDRNED(NPROC,0,NL-1,0,MXED,HVECT,IVECT,IBASE(INDPRO))
C----
C  COMPUTE MACROSCOPIC XS
C----
        MAXGAS=MAX(NGROUP**2,NBMIX**2,3*NBMIX*NNFI)
        CALL SETARA(IBASE,NPROC,ITYPRO)
        CALL SETARA(RBASE,NGROUP*NPROC,IXSREC)
        CALL SETARA(RBASE,NGROUP*NGROUP*NL,ISOSCT)
        CALL SETARA(RBASE,MAXGAS,IGAS)
        CALL SETARA(RBASE,NBMIX*MXXS*NGROUP,IGXS)
        CALL SETARA(RBASE,NBMIX*NGROUP*NGROUP*NL,ISCT)
        CALL SETARA(IBASE,MAX(NGROUP,NBMIX),INJJ)
        CALL SETARA(IBASE,MAX(NGROUP,NBMIX),IIJJ)
        CALL SETARA(IBASE,MAX(NGROUP,NBMIX),IIPOS)
        CALL SETARA(RBASE,NGROUP*NBMIX,ISCAT)
        CALL LIBDEN(IPLIB,NGROUP,NBISO,NBMIX,MXNL,MXED,NNFI,MXXS,
     >              ISONAM,IBASE(ISONFI),MIX,DEN,MASKK,MASK,
     >              LBASE(LMASKG),HVECT,IVECT,ITRANC,IBASE(IFGUPS),
     >              IBASE(IPXS),IBASE(IPSC),IBASE(INIFIS),
     >              IBASE(INOFIS),RBASE(IGAS),RBASE(IGXS),
     >              RBASE(ISCT),IBASE(INJJ),
     >              IBASE(IIJJ),IBASE(IIPOS),RBASE(ISCAT),
     >              ISTATE,ITSTMP,TMPDAY,RBASE(ISOSCT),NPROC,
     >              IBASE(INDPRO),IBASE(ITYPRO),RBASE(IXSREC))
        CALL RLSARA(RBASE(ISCAT))
        CALL RLSARA(IBASE(IIPOS))
        CALL RLSARA(IBASE(IIJJ))
        CALL RLSARA(IBASE(INJJ))
        CALL RLSARA(RBASE(ISCT))
        CALL RLSARA(RBASE(IGXS))
        CALL RLSARA(RBASE(IGAS))
        CALL RLSARA(RBASE(ISOSCT))
        CALL RLSARA(RBASE(IXSREC))
        CALL RLSARA(IBASE(ITYPRO))
        CALL RLSARA(IBASE(INDPRO))
      ENDIF
      CALL RLSARA (IBASE(IPSC))
      CALL RLSARA (IBASE(IPXS))
      CALL RLSARA (IBASE(IFGUPS))
      CALL RLSARA (LBASE(LMASKG))
      RETURN
C----
C  FORMAT
C----
 9000 FORMAT('LIBMIX: EXISTING MACROLIB HAS NGROUP=',I4,' ITRANC=',I4,
     >       ' NEW MACROLIB HAS NGROUP=',I4,' ITRANC=',I4)
 9001 FORMAT('LIBMIX: EXISTING MACROLIB HAS NBMIX=',I4,
     >       ' NEW MACROLIB HAS NBMIX=',I4)
      END
