*DECK PERVEC
      SUBROUTINE PERVEC(IPFLUR,IPFLUP,NBMIX ,NGROUP,MXNL  ,NIFISS,
     >                  MAXFIS,NUNKNO,NREGIO,LOLXSP,KEFFP ,LCOR  ,
     >                  LLIN ,
     >                  KEYFLX,MATCOD,VOLUME,
     >                  XSTOTP,XSTRAP,XSFISP,
     >                  XSFIXP,XSSCAP,XSFISR,
     >                  FUNKNO,FLUADJ,DELTFI,
     >                  QFDIR ,CORMAT,VARFLU,
     >                  VARCOR)
C
C---------------------------  PERVEC  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : PERVEC
C       USE      : COMPUTE VARIATION VECTORS TO BE MULTIPLIED
C                  BY GENERALIZED ADJOINT FLUXES
C       AUTHOR   : T. COURAU (2000-08-01)
C
C   2- ROUTINE PARAMETERS:
C    INPUT
C       IPFLUR   : POINTER TO THE FLUX                      I
C       IPFLUP   : POINTER TO THE PERTURBED FLUX            I
C       NBMIX    : MAXIMUM NUMBER OF MIXTURES               I
C       NGROUP   : NUMBER OF ENERGY GROUPS                  I
C       MXNL     : MAXIMUM LEGENDRE ORDER                   I
C                  = 1 ISOTROPIC COLLISION
C                  = 2 LINEARLY ANISOTROPIC COLLISION
C       NIFISS   : NUMBER OF FISSILE ISOTOPES               I
C       MAXFIS   : MAX(1,NIFISS)                            I
C       NUNKNO   : NUMBER OF UNKNOWN FOR THE FLUX           I
C       NREGIO   : NUMBER OF REGIONS                        I
C       LOLXSP   : CHECK IF CROSS SECTION TYPE IS           L(NBXS)
C                  TO BE TREATED
C                1 TOTAL, 2 NUSIGF, 3 FIXE, 4 CHI, 5  SCAD
C                6 TRANC, 7 FIXA, 8 FIXG, 9 NFTOT, 10 EFISS
C       LCOR     : FLAG FOR HIGH ORDER FLUX                 L
C                  COMPONENT CORRECTION
C       LLIN    : FLAG TO COMPUTE SENSITIVITY COEFFICIENT   L
C                 IF (LLIN) 2nd ORDERS TERM ARE NOT COMPUTED
C                 IN PERTURBATION FORMULAS
C       KEYFLX   : FLUX ELEMENTS IN UNKNOWN SYSTEM.         I(NREGIO)
C       MATCOD   : MATERIAL CODE IN REGIONS.                I(NREGIO)
C       VOLUME   : VOLUME OF REGIONS. DIMENSION             R(NREGIO)
C       XSTOT    : TOTAL CROSS SECTION OF MIXTURE           R(NBMIX,
C                                                             NGROUP)
C       XSTRA    : TRANSPORT CORRECTION XS OF MIXTURE       R(NBMIX,
C                                                             NGROUP)
C       XSFIS    : FISSION CROSS SECTION OF MIXTURE         R(NBMIX,
C                                                             MAXFIS,
C                                                             NGROUP,3)
C       XSFIX    : FIXE SOURCES                             R(NBMIX,
C                                                             NGROUP,3)
C       XSSCA    : SCATT X-SECTION OF MIXTURE/GROUP         R(NBMIX,
C                                                             MXNL,
C                                                             NGROUP,
C                                                             NGROUP)
C       XEFISS   : AVERAGE ENERGY PER FISSION               R(NBMIX,
C                                                             MAXFIS)
C       QFDIR    : FISSION CROSS SECTION                    D(NREGIO,
C                                                             NIFISS)
C       DELTFI   : STORE VARIATION PARAMATERS               R(NREGIO,
C                                                             NGROUP,3)
C       CORMAT   : HIGH ORDER COMPONENT                     R(NREGIO,
C                                                             NBMIX,
C                                                             NGROUP)
C                  CORRECTION MATRIX
C    INPUT/OUTPUT
C       KEFFP    : PERTURBED K-EFFECTIVE                    R
C       VARFLU   : STORE VECTOR TO BE MULTILIED             R(NREGIO,
C                                                                NGROUP)
C                  BY GENERALIZED PSEUDO ADJOINT FLUXES
C       VARCOR   : STORE VECTOR TO BE MULTILIED             R(NREGIO,
C                                                             NGROUP)
C                  BY GENERALIZED ADJOINT FLUXES
C                  (HIGH ORDER COMPONENT)
C---------------------------   PERVEC  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN,NBXS
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NBXS=10,NAMSBR='PERVEC')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPFLUR,IPFLUP,NBMIX,NGROUP,MXNL,NIFISS,MAXFIS,NUNKNO,
     >            NREGIO ,KEYFLX(NREGIO), MATCOD(NREGIO)
      REAL        VOLUME(NREGIO) ,XSTOTP(NBMIX,NGROUP),
     >            XSTRAP(NBMIX,NGROUP),XSFISP(NBMIX,MAXFIS,NGROUP,3),
     >            XSFIXP(NBMIX,NGROUP,3),
     >            XSSCAP(NGROUP,NBMIX,MXNL,NGROUP),
     >            XSFISR(NBMIX,MAXFIS,NGROUP,3),
     >            FUNKNO(NUNKNO,NGROUP),DELTFI(NREGIO,NGROUP,3),
     >            VARFLU(NREGIO,NGROUP),CORMAT(NREGIO,NBMIX,NGROUP),
     >            VARCOR(NREGIO,NGROUP),FLUADJ(NUNKNO,NGROUP),KEFFP
      LOGICAL     LOLXSP(NBXS),LCOR,LLIN
      DOUBLE PRECISION DLBDA,QFDIR(NREGIO,NIFISS)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
C----
C LOCAL PARAMETERS
C---
      INTEGER     IL,IGR,JGR,IFISS,IREG,IBM,IFPOS,IPFLUX,IMIX
      REAL        REFKEF
      CHARACTER   CGRPNM*12
      DOUBLE PRECISION DNUM,DNUMDP,DDEN,ACCUMU
C---
C READ FLUX
C--
      IF (IPFLUP.NE.0) THEN
       IPFLUX=IPFLUP
      ELSE
       IPFLUX=IPFLUR
      ENDIF
      CALL XDRSET(FUNKNO,NGROUP*NUNKNO,0.0)
      CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMUP)
      CALL FLUGFL(IPFLUX,NGROUP,NUNKNO,FUNKNO)
      CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMDN)
      CALL XDRSET(DELTFI,NREGIO*NGROUP*3,0.0)
C---
C READ CORRECTION DUE TO FLUX-1
C---
      CALL XDRSET(CORMAT,NREGIO*NBMIX*NGROUP,0.0)
      IF (LCOR) THEN
       CALL LCMSIX(IPFLUR,'FLUX1-CORR  ',ILCMUP)
       DO 10 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMGET(IPFLUR,CGRPNM,CORMAT(1,1,IGR))
 10    CONTINUE
       CALL LCMSIX(IPFLUR,'FLUX1-CORR  ',ILCMDN)
      ENDIF
C---
C  FISSION SOURCE TERM COMPUTATION
C--
      CALL XDRDBL(QFDIR,NREGIO*NIFISS,0.D0)
      DO 100 IGR=1,NGROUP
        DO 101 IREG=1,NREGIO
          IBM=MATCOD(IREG)
          IFPOS=KEYFLX(IREG)
          IF(IBM .GT. 0) THEN
           DO 102 IFISS=1,NIFISS
             IF (LLIN) THEN
              QFDIR(IREG,IFISS)=QFDIR(IREG,IFISS)+FUNKNO(IFPOS,IGR)*
     >              XSFISR(IBM,IFISS,IGR,1)
             ELSE
              QFDIR(IREG,IFISS)=QFDIR(IREG,IFISS)+FUNKNO(IFPOS,IGR)*
     >             (XSFISR(IBM,IFISS,IGR,1)+XSFISP(IBM,IFISS,IGR,1))
             ENDIF
 102       CONTINUE
          ENDIF
 101    CONTINUE
 100  CONTINUE
      DO 110 IGR=1,NGROUP
        DO 111 IREG=1,NREGIO
          IBM=MATCOD(IREG)
          IFPOS=KEYFLX(IREG)
          IF(IBM .GT. 0) THEN
           ACCUMU=0.D0
           DO 112 IFISS=1,NIFISS
            ACCUMU=ACCUMU+QFDIR(IREG,IFISS)*XSFISR(IBM,IFISS,IGR,3)
 112       CONTINUE
          ENDIF
          DELTFI(IREG,IGR,1)=REAL(ACCUMU*VOLUME(IREG))
 111    CONTINUE
 110  CONTINUE
C----
C VARIATION OF FISSION TERM COMPUTATION
C----
      IF (LOLXSP(2)) THEN
       CALL XDRDBL(QFDIR,NREGIO*NIFISS,0.D0)
       DO 200 IGR=1,NGROUP
        DO 201 IREG=1,NREGIO
          IBM=MATCOD(IREG)
          IFPOS=KEYFLX(IREG)
          DO 202 IFISS=1,NIFISS
           IF(IBM.GT.0 .AND. XSFISP(IBM,IFISS,IGR,1).NE.0.0 ) THEN
             QFDIR(IREG,IFISS)=QFDIR(IREG,IFISS)+
     >                         FUNKNO(IFPOS,IGR)*XSFISP(IBM,IFISS,IGR,1)
           ENDIF
 202      CONTINUE
 201    CONTINUE
 200   CONTINUE
       DO 210 IGR=1,NGROUP
        DO 211 IREG=1,NREGIO
          IBM=MATCOD(IREG)
          IFPOS=KEYFLX(IREG)
          IF(IBM .GT. 0) THEN
           DO 212 IFISS=1,NIFISS
            DELTFI(IREG,IGR,2)=DELTFI(IREG,IGR,2)
     >                        +QFDIR(IREG,IFISS)*XSFISR(IBM,IFISS,IGR,3)
 212       CONTINUE
          ENDIF
          DELTFI(IREG,IGR,2)=DELTFI(IREG,IGR,2)*VOLUME(IREG)
 211    CONTINUE
 210   CONTINUE
      ENDIF
C----
C VARIATION OF DIFFUSION TERM COMPUTATION
C----
      IF (LOLXSP(4)) CALL XABORT(NAMSBR//': CHI MUST NOT BE PERTURBED')
      IF (LOLXSP(5) .OR. LOLXSP(6) ) THEN
       IL=1
       DO 300 IGR=1,NGROUP
        DO 301 IREG=1,NREGIO
         IBM=MATCOD(IREG)
         IFPOS=KEYFLX(IREG)
         DO 302 JGR=1,NGROUP
           IF(IBM.GT.0) THEN
             IF (IGR.NE.JGR) THEN
              DELTFI(IREG,IGR,3)=DELTFI(IREG,IGR,3)-VOLUME(IREG)*
     >        FUNKNO(IFPOS,JGR)*XSSCAP(JGR,IBM,IL,IGR)
             ELSE
              DELTFI(IREG,IGR,3)=DELTFI(IREG,IGR,3)-VOLUME(IREG)*
     >        FUNKNO(IFPOS,JGR)*(XSSCAP(JGR,IBM,IL,IGR)-XSTRAP(IBM,IGR))
             ENDIF
           ENDIF
 302     CONTINUE
 301    CONTINUE
 300   CONTINUE
      ENDIF
C---
C VARIATION OF TOTAL AND TRANC  XS
C---
      IF (LOLXSP(1) .OR. LOLXSP(6)) THEN
        DO 500 IGR=1,NGROUP
         DO 501 IREG=1,NREGIO
         IBM=MATCOD(IREG)
         IFPOS=KEYFLX(IREG)
         IF(IBM.GT.0) DELTFI(IREG,IGR,3)=DELTFI(IREG,IGR,3)+VOLUME(IREG)
     >      *FUNKNO(IFPOS,IGR)*(XSTOTP(IBM,IGR)-XSTRAP(IBM,IGR))
 501    CONTINUE
 500   CONTINUE
      ENDIF
C---
C VARIATION OF XSTOT-TRANC ASSOCIATED TO VARCOR
C---
      CALL XDRSET(VARCOR,NREGIO*NGROUP,0.0)
      IF (LOLXSP(1) .OR. LOLXSP(6)) THEN
       DO 550 IGR=1,NGROUP
        DO 551 IREG=1,NREGIO
         IFPOS=KEYFLX(IREG)
         ACCUMU=0.D0
         DO 552 IMIX=1,NBMIX
          ACCUMU=ACCUMU+
     >     (XSTOTP(IMIX,IGR)-XSTRAP(IMIX,IGR))*CORMAT(IREG,IMIX,IGR)
 552     CONTINUE
         VARCOR(IREG,IGR)=REAL(ACCUMU*VOLUME(IREG)*FUNKNO(IFPOS,IGR))
 551    CONTINUE
 550   CONTINUE
      ENDIF
C----
C DLAMBDA COMPUTATION
C----
      DNUM=0.D0
      DNUMDP=0.D0
      DDEN=0.D0
      IPFLUX=IPFLUR
      CALL LCMGET(IPFLUX,'K-EFFECTIVE',REFKEF)
      CALL XDRSET(FUNKNO,NGROUP*NUNKNO,0.0)
      CALL LCMSIX(IPFLUX,'PSEUDOADJ   ',ILCMUP)
      CALL FLUGFL(IPFLUX,NGROUP,NUNKNO,FUNKNO)
      CALL LCMSIX(IPFLUX,'PSEUDOADJ   ',ILCMDN)
      IF (LCOR) THEN
       CALL XDRSET(FLUADJ,NGROUP*NUNKNO,0.0)
       CALL LCMSIX(IPFLUX,'FLUXADJ     ',ILCMUP)
       CALL FLUGFL(IPFLUX,NGROUP,NUNKNO,FLUADJ)
       CALL LCMSIX(IPFLUX,'FLUXADJ     ',ILCMDN)
      ENDIF
      DO 600 IGR=1,NGROUP
        DO 601 IREG=1,NREGIO
         IFPOS=KEYFLX(IREG)
         DNUM=DNUM+FUNKNO(IFPOS,IGR)*
     >            (DELTFI(IREG,IGR,3)-DELTFI(IREG,IGR,2)/REFKEF)
         IF (LCOR) DNUMDP=DNUMDP+FLUADJ(IFPOS,IGR)*VARCOR(IREG,IGR)
         DDEN=DDEN+FUNKNO(IFPOS,IGR)*DELTFI(IREG,IGR,1)
 601    CONTINUE
 600  CONTINUE
      DLBDA=(DNUM+DNUMDP)/DDEN
      CALL XDRSET(VARFLU,NREGIO*NGROUP,0.0)
      DO 700 IGR=1,NGROUP
        DO 701 IREG=1,NREGIO
         VARFLU(IREG,IGR)=DELTFI(IREG,IGR,2)/REFKEF
     >                   +DLBDA*DELTFI(IREG,IGR,1)-DELTFI(IREG,IGR,3)
 701    CONTINUE
 700  CONTINUE
      KEFFP=1.0D0/(1.0D0/REFKEF+DLBDA)
      RETURN
      END
