*DECK SADCOR
      SUBROUTINE SADCOR(IPFLUX,IPSYS ,IPMACR,IPTRK ,IPBTF,
     >                  NUNKNO,CDOOR ,IPRINT,NGROUP,NIFISS,
     >                  NBMIX ,NREG  ,MXNL  ,LEAKSW,ITRANC,
     >                  TITLE ,LNORM ,LALBS ,LR2   ,IPHASE,
     >                  ISCATR,KNORM ,CNAME ,ITPIJ ,MATCOD,
     >                  VOLUME,KEYFLX,FUNKNO,
     >                  SUNKNO,SOURCE,SIGMA,
     >                  CORMAT,SFUNKN)
C
C---------------------------  SADCOR  ---------------------------------
C
C 1- PROGRAMME STATISTICS:
C
C     NAME     :  SADCOR
C     USE      :  COMPUTE CORRECTION DUE TO HIGH ORDER FLUX COMPONENT
C     AUTHOR   :  T. COURAU
C
C 2- ROUTINE PARAMETERS:
C   INPUT
C     IPFLUX   : POINTER TO THE FLUX SOLUTION            I
C     IPSYS    : POINTER TO THE PIJ                      I
C     IPTRK    : POINTER TO THE TRACKING                 I
C     IPMACR   : POINTER TO THE MACROLIB                 I
C     IPBTF    : BINARY TRACKING FILE                    I
C     NUNKNO   : NUMBER OF UNKNOWNS                      I
C     CDOOR    : NAME OF THE PIJ CALCULATION DOOR        C*12
C     IPRINT   : PRINT SELECTION FOR SAD MODULE:         I
C     NGROUP   : NUMBER OF GROUPS                        I
C     NIFISS   : NUMBER OF FISSILE ISOTOPES              I
C     NBMIX    : NUMBER OF MIXTURES                      I
C     NREG     : NUMBER OF REGIONS                       I
C     NL       : NUMBER OF LEGENDRE ORDERS               I
C     LEAKSW   : LEAKAGE SWITCH                          L
C     ITRANC   : TYPE OF TRANSPORT CORRECTION            I
C     TITLE    : TITLE OF THE DRAGON CASE.               C*72
C     LNORM    : SWITCH FOR REMOVING LEAKAGE FROM CP     L
C     LALBS    : SWITCH FOR KEEPING THE PIS              L
C     LR2      : =.TRUE. TO STORE INFORMATION REQUIRED   L
C     IPHASE   : = -1 FOR ASS AND 0 FOR PIJ 1 FOR PIJK   I
C     ISCATR   : = 0 SCATTERING REDUCED/1 DIRECT         I
C     KNORM    : TYPE OF PIJ NORMALIZATION               I
C     CNAME    : NAME USED TO SAVE ADDITIONAL PIJ DATA   C*12
C     ITPIJ    : TYPE OF CP AVAILABLE                    I
C     MATCOD   : MATERIAL PER REGION                     I(NREG)
C     VOLUME   : VOLUME OF REGION                        R(NREG)
C     KEYFLX   : AVERAGE FLUX POSITION PER REGION        I(NREG)
C   OUTUT
C     CORMAT   : CORRECTION MATRIX                       R(NREG,NBMIX,
C                                                          NGROUP)
C   SCRATCH STORAGE
C     FUNKNO   : UNKNOWN VECTOR SOLVED FOR.              R(NUNKNO,
C                                                          NGROUP)
C     SUNKNO   : SOURCE FOR SYSTEM OF UNKNOWN.           R(NUNKNO,
C                                                          NGROUP)
C     SFUNKN   : TEMPORARY SOURCE FOR SYSTEM             R(NUNKNO,
C                                                          NGROUP)
C     SOURCE   : TEMPORARY SOURCE VECTOR                 R(NUNKNO)
C     SIGMA    : STORE XS TOT AND TRANC                  R(NBMIX,2)
C
C
C
C---------------------------   SADCOR  --------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='SADCOR')
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE(1)
      COMMON       RBASE
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      IQTOTL,IXSCHI,IXSSCAT,ISIGS0,IJJ,NJJ,IPOS
C---
C INPUT/OUTPUT VARIABLES
C---
      INTEGER  IPFLUX,IPSYS,IPMACR,IPTRK,IPBTF,NREG,NUNKNO,IPRINT,
     >         IREG,NIFISS,ITPIJ,
     >         NBMIX,MXNL,ITRANC,IPHASE,ISCATR,KNORM,NGROUP,
     >         MATCOD(NREG),KEYFLX(NREG)
      LOGICAL  LNORM ,LALBS ,LR2,LEAKSW
      CHARACTER CDOOR*12,CNAME*12,TITLE*72
      REAL     VOLUME(NREG),SIGMA(NBMIX,2),
     >         FUNKNO(NUNKNO,NGROUP), SUNKNO(NUNKNO,NGROUP),
     >         SOURCE(NUNKNO), CORMAT(NREG,NBMIX,NGROUP),
     >         SFUNKN(NUNKNO,NGROUP)
C----
C LOCAL VARIABLES
C----
      INTEGER IXSSIT,IXSSI0,IPSYSM,IPSYSR,IPMAD,ILCMLN,ILCMTY,IGR,
     >        IDIR,ITRANM,JMIX,IBM,ISCATP
      REAL    ALFACOR,KEFF
      CHARACTER CGRPNM*12
      DOUBLE PRECISION FACTNO
C---
C CHECK FOR BINARY TRACKING FILE
C---
      IF (CDOOR(1:5) .NE.'SYBIL'.AND. CDOOR(1:5).NE.'EXCEL') CALL
     > XABORT(NAMSBR//': CDOOR MUST BE SYBIL, EXCELL OR EXCELT')
      IF (CDOOR(1:5).EQ.'EXCEL'.AND.IPBTF.EQ.0) CALL XABORT(NAMSBR
     >     //': BINARY TRACKING FILE REQUIRED WITH EXCELL OR EXCELT')
C---
C COMPUTE REFERENCE  COLLISION PROBABILITIES:
C                    IF OPTION SKIP IPSYSR=IPSYS
C---
      ISCATP=1
      IF (ISCATR.EQ.1) THEN
       IPSYSR=IPSYS
      ELSE
       CALL LCMOP(IPSYSR,'PIJ_REFERENC',0,1,0)
       CALL SETARA(RBASE,NBMIX+1,IXSSIT)
       CALL SETARA(RBASE,(NBMIX+1)*MXNL,IXSSI0)
       CALL ASMDRV(IPSYSR ,IPTRK ,IPMACR,IPBTF,CDOOR ,IPRINT  ,
     >             NGROUP ,NBMIX, NREG  ,MXNL  ,LEAKSW,ITRANC,
     >             TITLE ,LNORM ,LALBS ,LR2   ,IPHASE,ISCATP,
     >             KNORM ,CNAME ,MATCOD  ,VOLUME  ,RBASE(IXSSIT),
     >             RBASE(IXSSI0))
       CALL RLSARA(RBASE(IXSSI0))
       CALL RLSARA(RBASE(IXSSIT))
      ENDIF
C----
C READ FLUX AND  COMPUTE ASSOCIATED SOURCE
C----
      CALL LCMGET(IPFLUX,'K-EFFECTIVE',KEFF)
      FACTNO=1.0D0/KEFF
      CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMUP)
      CALL FLUGFL(IPFLUX,NGROUP,NUNKNO,FUNKNO)
      CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMDN)
      CALL SETARA(RBASE,NBMIX*MAX(NIFISS,1),IXSCHI)
      CALL SETARA(RBASE,2*NREG*MAX(NIFISS,1),IQTOTL)
      CALL SETARA(RBASE,NBMIX*NGROUP+1,IXSSCAT)
      CALL SETARA(RBASE,NBMIX+1,ISIGS0)
      CALL SETARA(IBASE,NBMIX+1,IJJ)
      CALL SETARA(IBASE,NBMIX+1,NJJ)
      CALL SETARA(IBASE,NBMIX+1,IPOS)
      CALL SADSCE(IPMACR,IPSYSR,NGROUP,NBMIX ,NREG  ,NUNKNO,
     >            NIFISS,MATCOD,KEYFLX,FACTNO,ITPIJ ,SUNKNO,
     >            FUNKNO,RBASE(IXSCHI),RBASE(IQTOTL),
     >            RBASE(IXSSCAT),RBASE(ISIGS0),IBASE(IJJ)   ,
     >            IBASE(NJJ)    ,IBASE(IPOS))
      CALL RLSARA(IBASE(IJJ))
      CALL RLSARA(IBASE(NJJ))
      CALL RLSARA(IBASE(IPOS))
      CALL RLSARA(RBASE(ISIGS0))
      CALL RLSARA(RBASE(IXSSCAT))
      CALL RLSARA(RBASE(IXSCHI))
      CALL RLSARA(RBASE(IQTOTL))
      ALFACOR=-0.001
      IDIR=0
      CALL XDRSET(CORMAT,NREG*NBMIX*NGROUP,0.0)
      DO 5 JMIX=1,NBMIX
C---
C MODIFY MACROLIB
C---
       CALL LCMOP(IPMAD,'MACR_MODIFTR',0,1,0)
       CALL LCMEQU(IPMACR,IPMAD)
       CALL XDRSET(SFUNKN,NUNKNO*NGROUP,0.0)
       DO 10 IGR=1,NGROUP
        CALL XDRSET(SIGMA,2*NBMIX,0.0)
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMAD,CGRPNM,ILCMUP)
        CALL LCMLEN(IPMAD,'TOTAL       ',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. 0) CALL XABORT(NAMSBR//
     >    ': READ ERROR FOR RECORD-> TOTAL')
        CALL LCMGET(IPMAD,'TOTAL       ',SIGMA(1,1))
        IF (ITRANC.GT.0)  CALL LCMGET(IPMAD,'TRANC       ',SIGMA(1,2))
        DO 20 IREG=1,NREG
           IF (MATCOD(IREG) .EQ. JMIX) SFUNKN(KEYFLX(IREG),IGR)=ALFACOR*
     >         FUNKNO(KEYFLX(IREG),IGR)*(SIGMA(JMIX,1)-SIGMA(JMIX,2))
 20     CONTINUE
        SIGMA(JMIX,2)=SIGMA(JMIX,2)+
     >       (SIGMA(JMIX,1)-SIGMA(JMIX,2))*ALFACOR
        CALL LCMPUT(IPMAD,'TRANC       ',NBMIX,2,SIGMA(1,2))
        CALL LCMSIX(IPMAD,CGRPNM,ILCMDN)
 10    CONTINUE
       ITRANM=1
C---
C COMPUTE MODIFIED  COLLISION PROBABILITIES
C---
       CALL LCMOP(IPSYSM,'PIJ_MODIFTR ',0,1,0)
       CALL SETARA(RBASE,NBMIX+1,IXSSIT)
       CALL SETARA(RBASE,(NBMIX+1)*MXNL,IXSSI0)
       CALL ASMDRV(IPSYSM ,IPTRK ,IPMAD,IPBTF,CDOOR ,IPRINT  ,
     >            NGROUP  ,NBMIX,NREG  ,MXNL  ,LEAKSW,ITRANM,
     >            TITLE ,LNORM ,LALBS ,LR2   ,IPHASE,ISCATP,
     >            KNORM ,CNAME ,MATCOD  ,VOLUME  ,RBASE(IXSSIT),
     >            RBASE(IXSSI0))
       CALL RLSARA(RBASE(IXSSI0))
       CALL RLSARA(RBASE(IXSSIT))
C---
C COMPUTE (P+DELTA_P).DELTA_SIGMA.FI
C---
       DO 50 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPSYSM,CGRPNM,ILCMUP)
        CALL TRFICF(IPSYSM,IPTRK ,IPBTF ,IPRINT,IDIR  ,NREG,
     >            NUNKNO,MATCOD,VOLUME,KEYFLX,
     >            SOURCE,SFUNKN(1,IGR),TITLE)
        DO 60 IREG=1,NREG
           CORMAT(IREG,JMIX,IGR)=SOURCE(KEYFLX(IREG))
 60     CONTINUE
        CALL LCMSIX(IPSYSM,CGRPNM,ILCMDN)
 50   CONTINUE
C----
C COMPUTE DELTA_P
C----
       CALL LCMULT(IPSYSM,-1.0)
       CALL LCMADD(IPSYSR,IPSYSM)
C----
C COMPUTE DELTA_P.R.FI
C---
       DO 100 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPSYSM,CGRPNM,ILCMUP)
        CALL XDRSET (SOURCE,NUNKNO,0.0)
        CALL TRFICF(IPSYSM ,IPTRK ,IPBTF ,IPRINT,IDIR  ,NREG,
     >              NUNKNO,MATCOD,VOLUME,KEYFLX,
     >              SOURCE,SUNKNO(1,IGR),TITLE)
        DO 110 IREG=1,NREG
           CORMAT(IREG,JMIX,IGR)=
     >         CORMAT(IREG,JMIX,IGR)+SOURCE(KEYFLX(IREG))
 110    CONTINUE
        CALL LCMSIX(IPSYSM,CGRPNM,ILCMDN)
 100   CONTINUE
C---
C COMPUTE THE ROW OF THE MATRIX ASSOCIATED TO JMIX
C---
       CALL XDRSET(SIGMA,2*NBMIX,0.0)
       DO 300 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMGET(IPMACR,'TOTAL       ',SIGMA(1,1))
        IF (ITRANC.GT.0)  CALL LCMGET(IPMACR,'TRANC       ',SIGMA(1,2))
        DO 310 IREG=1,NREG
           IF ( (SIGMA(JMIX,1)-SIGMA(JMIX,2)) .NE.0.0) THEN
             CORMAT(IREG,JMIX,IGR)=-CORMAT(IREG,JMIX,IGR)/ALFACOR/
     >           (SIGMA(JMIX,1)-SIGMA(JMIX,2))/FUNKNO(KEYFLX(IREG),IGR)
           ELSEIF (CORMAT(IREG,JMIX,IGR).NE.0.0) THEN
             CALL XABORT( NAMSBR//'TOTAL CROSS SECTION VANISHES')
           ENDIF
 310    CONTINUE
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 300   CONTINUE
C---
C DELETE TEMPORARY LINKED LISTS
C---
       CALL LCMCL(IPSYSM,2)
       CALL LCMCL(IPMAD,2)
 5    CONTINUE
C-----
C PRINT CORRECTION MATRIX
C-----
      CALL LCMSIX(IPFLUX,'FLUX1-CORR  ',ILCMUP)
      DO 400 IGR=1,NGROUP
       WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
       CALL LCMPUT(IPFLUX,CGRPNM,NREG*NBMIX,2,CORMAT(1,1,IGR))
 400  CONTINUE
      CALL LCMSIX(IPFLUX,'FLUX1-CORR  ',ILCMDN)
C---
C DELETE TEMPORARY LINKED LIST
C---
      IF (ISCATR .EQ. 0) CALL LCMCL(IPSYSR,2)
      RETURN
      END
