*DECK SADFLU
      SUBROUTINE SADFLU(IPFLUX,IPSYS ,IPTMP,IPMACR,IPTRK  ,IPBTF ,
     >                  ITPIJ ,CXDOOR,COPTIO,IPRINT,LEAKSW,ITYPEC,
     >                  ISADJ ,ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,
     >                  NREGIO,NMERGE,NGCOND,NUNKNO,NIFISS,MXNL  ,
     >                  INITFL,B2    ,REFKEF,IXSFL,TITLE  ,
     >                  NAMGPA,NAMGAS,FUNKNO,SUNKNO,
     >                  MATCOD,VOLUME,KEYFLX,
     >                  IMERGE,IGCOND)
C
C---------------------------  SADFLU  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : SADFLU
C       USE      : COMPUTE THE GENERALIZED ADJOINT FLUXES ASSOCIATED
C                  WITH THE CONDENSATION & HOMOGENIZATION SOURCES
C       AUTHOR   : T. COURAU 2000/08/01
C
C   2- ROUTINE PARAMETERS:
C    INPUT/OUTPUT:
C      IPFLUX  : POINTER TO THE SOLUTION                   I
C      IPSYS   : POINTER TO THE PIJ                        I
C      IPTMP    : TEMPORARY POINTER TO THE GENERALIZED     I
C                 ADJOINT SOURCES
C      IPMACR  : POINTER TO THE MACROLIB                   I
C      IPTRK   : POINTER TO THE TRACKING                   I
C      IPBTF   : POINTER TO THE BINARY TRACKING FILE       I
C      ITPIJ   : TYPE OF CP AVAILABLE                      I
C                = 1 SCATT MOD PIJ (WIJ)
C                = 2 STAND. PIJ
C                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
C                = 4 STAND. PIJ+PIJK
C      CXDOOR  : NAME OF DOOR FOR FLUX SOLUTION            C*12
C      COPTIO  : CHARACTER FOR OPTION TYPE                 C*4
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES          I
C                = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      LEAKSW  : LEAKAGE SWITCH.                           L
C      ITYPEC  : TYPE OF FLUX EVALUATION                   I
C                = 0  SKIP THE FLUX CALCULATION
C                = 1  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE
C                = 2  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     FIXED DB2 BUCKLING
C                = 3  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     DB2 BUCKLING EVALUATION
C                = 4  DIRECT PROBLEM FISSION SOURCES/
C                     DB2 BUCKLING CONVERGENCE
C                = 5  DIRECT PROBLEM DB2 SOURCES
C                = 6  DIRECT PROBLEM FIXED SOURCES
C      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION       I
C                = 0 ONLY DIRECT FLUX
C                = 1 DIRECT AND PSEUDO ADJOINT FLUX
C                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
C                = 3 DIRECT, PSEUDO ADJOINT AND
C                    GENERALIZED PSEUDO-ADJOINT
C                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
C                    GENERALIZED PSEUDO-ADJOINT AND
C                    GENERALIZED ADJOINT
C      ILEAK   : METHOD USED TO INCLUDE LEAKAGE EFFECT     I
C                = 0 NO DB2 EFFECT
C                = 1 HOMOGENEOUS WITH SCATTERING MODIFIED
C                    CP MATRIX MULTIPLIED BY A PNL FACTOR
C                = 2 HOMOGENEOUS REDUCED CP MATRIX
C                    MULTIPLIED BY PNL FACTOR
C                    (DEFAULT FOR TYPE B AND L)
C                = 3 HOMOGENEOUS SIGS0-DB2 APPROXIMATION
C                = 4 HOMOGENEOUS ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
C                  AND FIXED B->5 OR
C                  SEARCH X->6 Y->7 Z->8 R->9 ALL->10
C      ICOPAR  : ITERATION PARAMETERS                      I(6)
C                IFRITR=ICOPAR(1)
C                IACITR=ICOPAR(2)
C                IREBAL=ICOPAR(3)
C                MAXINR=ICOPAR(4)
C                MAXOUT=ICOPAR(5)
C                ICONTM=ICOPAR(6)
C      EPSPAR  : ITERATION PRECISION                       R(NBEPS)
C                EPSINR=EPSPAR(1)
C                EPSOUT=EPSPAR(2)
C                EPSUNK=EPSPAR(3)
C                EPSGPA=EPSPAR(4)
C                CONGPA=EPSPAR(5)
C      NGROUP  : NUMBER OF GROUPS.                         I
C      NBMIX   : NUMBER OF MIXTURES.                       I
C      NREGIO  : NUMBER OF REGIONS.                        I
C      NMERGE  : NUMBER OF REGIONS MERGED                  I
C      NGCOND  : NUMBER OF GROUPS CONDENSED                I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.          I
C      NIFISS  : NUMBER OF FISSILE ISOTOPES.               I
C      MXNL    : MAXIMUM ANISOTROPY
C      INITFL  : FLUX INITIALIZATION OPTION                I
C                = 0 DEFAULT INITIALIZATION
C                = 1 TAKEN FROM FLXUNK
C                = 2 TAKEN FROM DATA STRUCTURE
C      B2      : INITIAL ESTIMATE OF THE BUCKLING FOR      R(4)
C                THE B2 SEARCH OR IMPOSED BUCKLING.
C      REFKEF  : TARGET K-EFFECTIVE FOR TYPE B OR L        D
C      IXSFL    : FLAG FOR THE XS TO BE TREATED            I(NNAM+1)
C                1 TOTAL, 2 TRANC, 3 NUSIGF, 4 NFTOT,
C                5 CHI, 6  SCAD
C      TITLE   : TITLE OF THE DRAGON CASE.                 C*72
C      NAMGPA  : NAME OF GENERALIZED PSEUDO-ADJOINT        C(2)*12
C      NAMGAS  : NAME OF GENERALIZED ADJOINT SOURCE        C*12
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.                R(NUNKNO,
C                                                            NGROUP)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.             R(NUNKNO,
C                                                            NGROUP)
C      MATCOD  : MATERIAL CODE IN REGIONS.                 I(NREGIO)
C      VOLUME  : VOLUME OF REGIONS. DIMENSION              R(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.          I(NREGIO)
C      IMERGE  : MERGED REGIONS POSITION                   I(NREGIO)
C      IGCOND  : LIMIT CONDENSED GROUPS                    I(NGCOND)
C
C   3- INTERNAL PARAMETERS:
C      NNAM    : NUMBER OF STANDARD XS TREATED             I
C
C---------------------------   SADFLU  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,NBEPS,ILCMUP,ILCMDN,NNAM
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NBEPS=5,ILCMUP=1,ILCMDN=2,NAMSBR='SADFLU',
     >            NNAM=6)
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPFLUX,IPSYS,IPTMP,IPMACR,IPTRK,IPBTF,ITPIJ,IPRINT,
     >            ITYPEC,ISADJ,ILEAK,ICOPAR(6),NGROUP,NBMIX,NREGIO,
     >            NMERGE,NGCOND,NUNKNO,NIFISS,MXNL,INITFL,
     >            MATCOD(NREGIO),KEYFLX(NREGIO),IMERGE(NREGIO),
     >            IGCOND(NGCOND),IXSFL(NNAM+1)
      CHARACTER   CXDOOR*12,COPTIO*4,TITLE*72,NAMGPA(2)*12,NAMGAS*12,
     >            NAMTMP*12
      LOGICAL     LEAKSW
      REAL        EPSPAR(NBEPS),B2(4),FUNKNO(NUNKNO,NGROUP),
     >            SUNKNO(NUNKNO,NGROUP),VOLUME(NREGIO)
      DOUBLE PRECISION REFKEF
      INTEGER     IFSUR
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   NAME(NNAM)*6,CGRPNM*12
      INTEGER     IPMAD,ITYCCT,INAM,IMRG,IGRC,ITYCC,IGRFIN,
     >            IGRDEB,IGROUP,IREG,IMIX,IL,JGROUP,JGRC,INITAD
      LOGICAL     LNILL
C-----
C GENERATE A TEMPORARY MACROLIB
C-----
      IFSUR=0
      CALL LCMOP(IPMAD,'MACRO+SADJ',0,2,0)
      CALL LCMEQU(IPMACR,IPMAD)
C-----
C GENERALIZED ADJOINT FLUX COMPUTATION
C----
      NAME(1)='TOTAL '
      NAME(2)='TRANC '
      NAME(3)='NUSIGF'
      NAME(4)='NFTOT '
      NAME(5)='CHI   '
      NAME(6)='ABS   '
      NAMGAS='TMPFIXG'
C-----
C SET FLUX PARAMETERS
C----
      INITAD=2
      COPTIO='B0  '
      REFKEF=1.0D0
      B2(1)=0.0
      B2(2)=0.0
      B2(3)=0.0
      B2(4)=0.0
C----
C IF LEAKAGE IMPOSE FIXED DB2 COMPUTATION
C----
      IF(ITYPEC .GE. 2 .AND. ITYPEC .LE. 5)THEN
        ITYCCT=2
      ELSE
        ITYCCT=ITYPEC
      ENDIF
C---
C LOOP OVER STANDARD CROSS SECTIONS
C---
      DO 1 INAM=1,NNAM
       IF (IXSFL(INAM).NE.1) GO TO 1
       DO 10 IMRG=1,NMERGE
        IGRFIN=0
        DO 20 IGRC=1,NGCOND
         ITYCC=ITYCCT
         CALL XDRSET(SUNKNO,NUNKNO*NGROUP,0.0)
         IGRDEB=IGRFIN+1
         IGRFIN=IGCOND(IGRC)
         DO 30 IGROUP=IGRDEB,IGRFIN
C--
C SOURCE READ AND STORED IN SUNKNO
C--
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
          CALL LCMSIX(IPTMP,CGRPNM,ILCMUP)
          NAMGPA(2)=NAME(INAM)
          CALL LCMGET(IPTMP,NAMGPA(2),SUNKNO(1,IGROUP))
          DO 40 IREG=1,NREGIO
            IMIX=MATCOD(IREG)
            IF(IMERGE(IREG).NE.IMRG.AND.IMIX.GT.0) THEN
             SUNKNO(IMIX,IGROUP)=0.0
            ENDIF
 40       CONTINUE
          CALL LCMSIX(IPTMP,CGRPNM,ILCMDN)
 30      CONTINUE
C---
C CHECK IF THE SOURCE VANISHES (LNILL=.TRUE.)
C---
         LNILL=.TRUE.
         DO 45 JGROUP=1,NGROUP
          DO 46 IMIX=1,NBMIX
           LNILL=LNILL.AND.SUNKNO(IMIX,JGROUP).EQ.0.0
 46       CONTINUE
 45      CONTINUE
         IF (LNILL) THEN
           ITYCC=0
           GO TO 47
         ENDIF
C---
C SOURCE PRINTED ON THE TEMPORARY MACROLIB
C---
         DO 50 JGROUP=1,NGROUP
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') JGROUP,NGROUP
          CALL LCMSIX(IPMAD,CGRPNM,ILCMUP)
          CALL LCMPUT(IPMAD,NAMGAS,NBMIX,2,SUNKNO(1,JGROUP))
          CALL LCMSIX(IPMAD,CGRPNM,ILCMDN)
 50      CONTINUE
 47      CONTINUE
C---
C GENERALIZED ADJOINT FLUX COMPUTATION
C---
         WRITE(NAMGPA(1),'(A6,2I2)') NAME(INAM),IGRC,IMRG
         NAMTMP=NAMGPA(1)
         NAMGPA(2)='A'//NAMTMP(2:12)
         IF (IPRINT.GE.1) THEN
           WRITE(IOUT,6010) '  SOURCE'
           WRITE(IOUT,6014)
           WRITE(IOUT,6016) NAMGPA(1)
           IF (LNILL) WRITE(IOUT,6017)
         ENDIF
         CALL FLUDRV(IPFLUX,IPSYS ,IPMAD ,IPTRK ,IPBTF ,ITPIJ ,
     >               CXDOOR,COPTIO,IPRINT,LEAKSW,ITYCC ,ISADJ,
     >               ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREGIO,
     >               NUNKNO,NIFISS,MXNL  ,INITAD,B2    ,REFKEF,
     >               TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >               FUNKNO,SUNKNO,MATCOD,VOLUME,KEYFLX)
 20     CONTINUE
 10    CONTINUE
 1    CONTINUE
C---
C LOOP OVER SCATTERING CROSS SECTIONS
C---
      IF (IXSFL(NNAM+1).EQ.1) THEN
       IL=1
c$$$       DO 100 IL=1,MXNL
        DO 105 JGRC=1,NGCOND
         DO 110 IMRG=1,NMERGE
          IGRFIN=0
          DO 120 IGRC=1,NGCOND
           ITYCC=ITYCCT
           CALL XDRSET(SUNKNO,NUNKNO*NGROUP,0.0)
           IGRDEB=IGRFIN+1
           IGRFIN=IGCOND(IGRC)
C--
C SOURCE READ AND STORED IN SUNKNO
C--
           DO 130 IGROUP=IGRDEB,IGRFIN
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
            CALL LCMSIX(IPTMP,CGRPNM,ILCMUP)
            WRITE(NAMGPA(2),'(A4,2I2)')  'SCAD',IL-1,JGRC
            CALL LCMGET(IPTMP,NAMGPA(2),SUNKNO(1,IGROUP))
            DO 140 IREG=1,NREGIO
             IMIX=MATCOD(IREG)
             IF(IMERGE(IREG).NE.IMRG.AND.IMIX.GT.0) THEN
              SUNKNO(IMIX,IGROUP)=0.0
             ENDIF
 140        CONTINUE
            CALL LCMSIX(IPTMP,CGRPNM,ILCMDN)
 130       CONTINUE
C---
C CHECK IF THE SOURCE VANISHES (LNILL=.TRUE.)
C---
           LNILL=.TRUE.
           DO 145 JGROUP=1,NGROUP
            DO 146 IMIX=1,NBMIX
             LNILL=LNILL.AND.SUNKNO(IMIX,JGROUP).EQ.0.0
 146        CONTINUE
 145       CONTINUE
           IF (LNILL) THEN
            ITYCC=0
            GO TO 147
           ENDIF
C---
C SOURCE PRINTED ON THE TEMPORARY MACROLIB
C---
           DO 150 JGROUP=1,NGROUP
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') JGROUP,NGROUP
            CALL LCMSIX(IPMAD,CGRPNM,ILCMUP)
            CALL LCMPUT(IPMAD,NAMGAS,NBMIX,2,SUNKNO(1,JGROUP))
            CALL LCMSIX(IPMAD,CGRPNM,ILCMDN)
 150       CONTINUE
 147       CONTINUE
C---
C GENERALIZED ADJOINT FLUX COMPUTATION
C---
           WRITE(NAMGPA(1),'(A4,4I2)')'SCAD',IL-1,JGRC,IGRC,IMRG
           NAMTMP=NAMGPA(1)
           NAMGPA(2)='A'//NAMTMP(2:12)
           IF (IPRINT.GE.1) THEN
            WRITE(IOUT,6010) '  SOURCE'
            WRITE(IOUT,6014)
            WRITE(IOUT,6016) NAMGPA(1)
            IF (LNILL) WRITE(IOUT,6017)
           ENDIF
           CALL FLUDRV(IPFLUX,IPSYS ,IPMAD ,IPTRK ,IPBTF ,ITPIJ ,
     >                 CXDOOR,COPTIO,IPRINT,LEAKSW,ITYCC ,ISADJ,
     >                 ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREGIO,
     >                 NUNKNO,NIFISS,MXNL  ,INITAD,B2    ,REFKEF,
     >                 TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >                 FUNKNO,SUNKNO,MATCOD,VOLUME,KEYFLX)
 120      CONTINUE
 110     CONTINUE
 105    CONTINUE
      ENDIF
      CALL LCMCL(IPMAD,2)
      RETURN
 6010 FORMAT(////1X,'SOLUTION TO TRANSPORT EQUATION ',/
     > ' CALCULATION TYPE            =',2X,A8)
 6014 FORMAT(
     > ' PROBLEM TYPE                =   GEN-ADJ')
 6016 FORMAT(
     > ' SOURCE TYPE                 =',A12)
 6017 FORMAT(1X,'*** WARNING *** GENERALIZED ADJOINT SOURCE',
     >  ' VANISHES -> NO FLUX COMPUTATION')
      END
