*DECK SADINI
      SUBROUTINE SADINI(IPFLUX,IPSYS ,IPMACR,IPTRK ,IPBTF ,ITPIJ ,
     >                  CXDOOR,COPTIO,IPRINT,INITFL,LEAKSW,ITYPEC,
     >                  ISADJ ,ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,
     >                  NREGIO,NUNKNO,NIFISS,MXNL  ,B2    ,REFKEF,
     >                  TITLE ,NAMGPA,NAMGAS,
     >                  FUNKNO,SUNKNO,MATCOD,
     >                  VOLUME,KEYFLX)
C
C---------------------------  SADINI  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : SADINI
C       USE      : COMPUTE DIRECT AND ADJOINT FLUX
C       AUTHOR   : T. COURAU 2000/08/01
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      IPFLUX  : POINTER TO THE SOLUTION                   I
C      IPSYS   : POINTER TO THE PIJ                        I
C      IPMACR  : POINTER TO THE MACROLIB                   I
C      IPTRK   : POINTER TO THE TRACKING                   I
C      IPBTF   : POINTER TO THE BINARY TRACKING FILE       I
C      ITPIJ   : TYPE OF CP AVAILABLE                      I
C                = 1 SCATT MOD PIJ (WIJ)
C                = 2 STAND. PIJ
C                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
C                = 4 STAND. PIJ+PIJK
C      CXDOOR  : NAME OF DOOR FOR FLUX SOLUTION            C*12
C      COPTIO  : LEAKAGE TYPE                              C*4
C                'LKRD' (IMPOSED LEAKAGE ), 'B0' (B-0),
C                'P0' (P-0), 'B1' (B-1), 'P1' (P-1)
C                'B0TR' (B-0 WITH TRANSPORT CORRECTION).
C                COPTIO='B0' IS DEFAULT FOR TYPE B AND L
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES          I
C                = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      INITFL  : FLUX INITIALIZATION OPTION                I
C                = 0 DEFAULT INITIALIZATION
C      LEAKSW  : LEAKAGE SWITCH.                           L
C      ITYPEC  : TYPE OF FLUX EVALUATION                   I
C                = 0  SKIP THE FLUX CALCULATION
C                = 1  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE
C                = 2  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     FIXED DB2 BUCKLING
C                = 3  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     DB2 BUCKLING EVALUATION
C                = 4  DIRECT PROBLEM FISSION SOURCES/
C                     DB2 BUCKLING CONVERGENCE
C                = 5  DIRECT PROBLEM DB2 SOURCES
C                = 6  DIRECT PROBLEM FIXED SOURCES
C      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION       I
C                = 0 ONLY DIRECT FLUX
C                = 1 DIRECT AND PSEUDO ADJOINT FLUX
C                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
C                = 3 DIRECT, PSEUDO ADJOINT AND
C                    GENERALIZED PSEUDO-ADJOINT
C                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
C                    GENERALIZED PSEUDO-ADJOINT AND
C                    GENERALIZED ADJOINT
C      ILEAK   : METHOD USED TO INCLUDE LEAKAGE EFFECT     I
C                = 0 NO DB2 EFFECT
C                = 1 HOMOGENEOUS WITH SCATTERING MODIFIED
C                    CP MATRIX MULTIPLIED BY A PNL FACTOR
C                = 2 HOMOGENEOUS REDUCED CP MATRIX
C                    MULTIPLIED BY PNL FACTOR
C                    (DEFAULT FOR TYPE B AND L)
C                = 3 HOMOGENEOUS SIGS0-DB2 APPROXIMATION
C                = 4 HOMOGENEOUS ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
C                  AND FIXED B->5 OR
C                  SEARCH X->6 Y->7 Z->8 R->9 ALL->10
C      ICOPAR  : ITERATION PARAMETERS                      I(6)
C                IFRITR=ICOPAR(1)
C                IACITR=ICOPAR(2)
C                IREBAL=ICOPAR(3)
C                MAXINR=ICOPAR(4)
C                MAXOUT=ICOPAR(5)
C                ICONTM=ICOPAR(6)
C      EPSPAR  : ITERATION PRECISION                       R(NBEPS)
C                EPSINR=EPSPAR(1)
C                EPSOUT=EPSPAR(2)
C                EPSUNK=EPSPAR(3)
C                EPSGPA=EPSPAR(4)
C                CONGPA=EPSPAR(5)
C      NGROUP  : NUMBER OF GROUPS.                         I
C      NBMIX   : NUMBER OF MIXTURES.                       I
C      NREGIO  : NUMBER OF REGIONS.                        I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.          I
C      NIFISS  : NUMBER OF FISSILE ISOTOPES.               I
C      MXNL    : MAXIMUM ANISOTROPY                        I
C      B2      : INITIAL ESTIMATE OF THE BUCKLING FOR      R(4)
C                THE B2 SEARCH OR IMPOSED BUCKLING.
C      REFKEF  : TARGET K-EFFECTIVE FOR TYPE B OR L        D
C      TITLE   : TITLE OF THE DRAGON CASE.                 C*72
C      NAMGPA  : NAME OF GENERALIZED PSEUDO-ADJOINT        C(2)*12
C      NAMGAS  : NAME OF GENERALIZED ADJOINT SOURCE        C*12
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.                R(NUNKNO,
C                                                            NGROUP)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.             R(NUNKNO,
C                                                            NGROUP)
C      MATCOD  : MATERIAL CODE IN REGIONS.                 I(NREGIO)
C      VOLUME  : VOLUME OF REGIONS. DIMENSION              R(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.          I(NREGIO)
C
C---------------------------   SADINI  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,NBEPS,NSTATE
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NSTATE=20,NBEPS=5,NAMSBR='SADINI')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPFLUX,IPSYS,IPMACR,IPTRK,IPBTF,ITPIJ,IPRINT,ITYPEC,
     >            ITYCC,ISADJ,ILEAK,NGROUP,NBMIX,NREGIO,NUNKNO,NIFISS,
     >            MXNL,INITFL,ISTATE(NSTATE),ISADT
      CHARACTER   CXDOOR*12,COPTIO*4,TITLE*72,NAMGPA(2)*12,NAMGAS*12
      INTEGER     ICOPAR(6),MATCOD(NREGIO),KEYFLX(NREGIO)
      LOGICAL     LEAKSW
      REAL        EPSPAR(NBEPS),B2(4),FUNKNO(NUNKNO,NGROUP),
     >            SUNKNO(NUNKNO,NGROUP),VOLUME(NREGIO)
      DOUBLE PRECISION REFKEF
      INTEGER     IFSUR
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
C-----
C DATA
C-----
      CHARACTER   HTYPE(6)*8
      DATA        HTYPE
     >           /'   K-INF','   K-EFF','   K-EFF',
     >            'BUCKLING',' LEAKAGE','  SOURCE'/
C-----
C DIRECT FLUX COMPUTATION
C----
      IFSUR=0
      ITYCC=ITYPEC
      ISADT=ISADJ
      ISADJ=0
      IF (IPRINT .GE. 1) THEN
        WRITE(IOUT,6010) HTYPE(ITYCC)
        WRITE(IOUT,6011)
      ENDIF
      CALL FLUDRV(IPFLUX,IPSYS ,IPMACR,IPTRK ,IPBTF ,ITPIJ ,
     >            CXDOOR,COPTIO,IPRINT,LEAKSW,ITYCC ,ISADJ,
     >            ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREGIO,
     >            NUNKNO,NIFISS,MXNL  ,INITFL,B2    ,REFKEF,
     >            TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >            FUNKNO,SUNKNO,MATCOD,VOLUME,KEYFLX)
C-----
C ADJOINT FLUX COMPUTATION
C----
      ISADJ=ISADT
      IF(ITYPEC .GE. 2 .AND. ITYPEC .LE. 5) ITYCC=2
      IF (IPRINT .GE. 1) THEN
        WRITE(IOUT,6010) HTYPE(ITYCC)
        WRITE(IOUT,6014)
      ENDIF
      CALL FLUDRV(IPFLUX,IPSYS ,IPMACR,IPTRK ,IPBTF ,ITPIJ ,
     >            CXDOOR,COPTIO,IPRINT,LEAKSW,ITYCC ,ISADJ,
     >            ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREGIO,
     >            NUNKNO,NIFISS,MXNL  ,INITFL,B2    ,REFKEF,
     >            TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >            FUNKNO,SUNKNO,MATCOD,VOLUME,KEYFLX)
      CALL LCMPUT(IPFLUX,'EPS-CONVERGE',NBEPS,2,EPSPAR)
      CALL XDRSET(ISTATE,NSTATE,0)
      ISTATE(1)=ITYPEC
      ISTATE(2)=ILEAK
      ISTATE(3)=ICOPAR(1)
      ISTATE(4)=ICOPAR(2)
      ISTATE(5)=ICOPAR(3)
      ISTATE(6)=ICOPAR(4)
      ISTATE(7)=ICOPAR(5)
      ISADJ=3
      ISTATE(8)=ISADJ
      ISTATE(9)=NGROUP
      ISTATE(10)=NUNKNO
      ISTATE(11)=ICOPAR(6)
      ISTATE(12)=IFSUR
      CALL LCMPUT(IPFLUX,'STATE-VECTOR',NSTATE,1,ISTATE)
      RETURN
C---
C FORMAT
C---
 6010 FORMAT(////1X,'SOLUTION TO TRANSPORT EQUATION ',/
     > ' CALCULATION TYPE            =',2X,A8)
 6011 FORMAT(
     > ' PROBLEM TYPE                =   DIRECT')
 6014 FORMAT(
     > ' PROBLEM TYPE                =   ADJOINT')
      END
