*DECK SADPSA
      SUBROUTINE  SADPSA(IPTMP,NGCOND,NMERGE,NL    ,NREGIO,
     >                   NGROUP,ITRANC,NIFISS,NEDMAC,NNTOTF,
     >                   NTAUXT,NBMIX ,NNAM  ,NAMEAD,IMERGE,
     >                   IGCOND,NAME  ,INDICE,IXSFL ,MATCOD,
     >                   HVECT ,RATECM,SCATTD,SADXS ,XSSCAT,
     >                   FLAGMX,PCHI  ,PFIS  ,CHISUM,SASCAT,
     >                                               DASCAT)
C
C
C---------------------------  SADPSA  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : SADPSA
C      USE      : EVALUATE & PRINT GENERALIZED ADJOINT SOURCES
C      AUTHOR   : T. COURAU 2000/08/01
C      REMARK   : THIS ROUTINE IS INSPIRED FROM EDIPXS
C
C  2- ROUTINE PARAMETERS:
C    INPUT / OUTPUT
C      IPTMP   : TEMPORARY POINTER TO THE GENERALIZED   I
C                ADJOINT SOURCES
C      NGCOND   : NUMBER OF GROUPS CONDENSED            I
C      NMERGE   : NUMBER OF REGIONS MERGED              I
C      NL       : NUMBER OF LEGENDRE ORDERS             I
C      NREGIO   : NUMBER OF REGIONS                     I
C      NGROUP   : NUMBER OF GROUPS                      I
C      ITRANC   : TYPE OF TRANSPORT CORRECTION          I
C      NIFISS   : NUMBER OF FISSILE ISOTOPES            I
C      NEDMAC   : NUMBER OF ADITIONAL EDITION X-S       I
C      NNTOTF   : FLAG FOR NFTOT                        I
C      NTAUXT   : NUMBER OF REACTION RATE EDITS         I
C      NBMIX    : NUMBER OF MIXTURES                    I
C      NNAM     : NUMBER OF STANDARD XS TREATED         I
C      NAMEAD   : EXTRA EDIT NAMES                      I(2,NEDMAC)
C      IMERGE   : INDEX OF MERGED REGION                I(NREGIO)
C      IGCOND   : LIMIT CONDENSED GROUPS                I(NGCOND)
C      NAME     : LIST OF STANDARD XS CONSIDERED        C(NNAM)*6
C      INDICE   : INDEX OF THE STANDARD XS              I(NNAM)
C                        STORED IN RATECM
C      IXSFL    : FLAG FOR THE XS TO BE TREATED         I(NNAM+1)
C                1 TOTAL, 2 TRANC, 3 NUSIGF, 4 NFTOT,
C                5 CHI, 6  SCAD
C      MATCOD   : MATERIAL PER REGION                   I(NREGIO)
C      HVECT    : EXTRA EDIT NAMES                      C(NEDMAC)*6
C      RATECM   : AVERAGED REGION/GROUP X-S             D(0:NMERGE,
C                 = RATECM(*,1) = FLUXES                  NGCOND,NTAUXT)
C                 = RATECM(*,2) = TOTAL
C                 = RATECM(*,3) = ABS
C                 = RATECM(*,4) = FISSIONS
C                 = RATECM(*,5) = PRODUCTIONS
C                 = RATECM(*,6) = TOTAL LEAKAGES
C                 = RATECM(*,7) = TOTAL OUT OF GROUP SCATTERING
C                 = RATECM(*,8) = DIAGONAL SCATTERING X-S
C                 = RATECM(*,9) = CHI
C                 = RATECM(*,10) = WIMS TYPE TRANSPORT CORRECTION
C                 = RATECM(*,11) = X-DIRECTED LEAKAGES
C                 = RATECM(*,12) = Y-DIRECTED LEAKAGES
C                 = RATECM(*,13) = Z-DIRECTED LEAKAGES
C                 = RATECM(*,*,13+JJ) NUSIGF FOR JJ
C                 = RATECM(*,*,13+NIFISS+JJ) CHI FOR JJ
C                 = RATECM(*,*,13+2*NIFISS+JJ) NFTOT FOR JJ
C                 = RATECM(*,*,13+3*NIFISS+IL) SIGS FOR IL
C                 = RATECM(*,*,13+3*NIFISS+NL+IE) EDIT IE
C                 = RATECM(*,*,NTAUXT)= 1/V MERGE CONDENSED
C      SCATTD   : DOUBLE PRECISION SCATTERING RATES     D(NMERGE,NGCOND,
C                                                         NGCOND,NL)
C      SADXS    : GENERALIZED ADJOINT SOURCE            R(0:NBMIX,
C                                                         NGROUP,NNAM)
C                            FOR STANDARD XS
C      XSSCAT   : SCATTERING MATRIX                     R(NGROUP,NGROUP,
C                                                         NBMIX,NL)
C    WORK
C      FLAGMX   : CHECK IF A MIX HAS BEEN TREATED       I(NBMIX)
C      PCHI     : STORE SPECTRUM                        R(NBMIX,NIFISS,
C                                                         NGROUP)
C      PFIS     : STORE FISSION XS                      R(NBMIX,NIFISS,
C                                                         NGROUP)
C      CHISUM   : STORE CONDENSED SPECTRUM              D(NIFISS)
C      SASCAT   : GENERALIZED ADJOINT SOURCE            R(NBMIX,NGCOND,
C                 FOR SCATTERING XS                       NGROUP,NL)
C
C  3- INTERNAL PARAMETERS:
C
C      IOUT    : OUTPUT UNIT = 6                    I
C      CGRPNM  : GROUP INDEX NAME FOR LCM           C*12
C
C---------------------------   SADPSA  --------------------------------
C
      IMPLICIT          NONE
      INTEGER           IOUT,ILCMUP,ILCMDN
      CHARACTER         NAMSBR
      PARAMETER        (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='SADPSA')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER           IPTMP,NGCOND,NMERGE,NL,NREGIO,
     >                  IGCOND(NGCOND),NGROUP,ITRANC,IMERGE(NREGIO),
     >                  NNAM,NIFISS,NEDMAC,NNTOTF,NTAUXT,
     >                  NAMEAD(2,NEDMAC),NBMIX,MATCOD(NREGIO),
     >                  FLAGMX(NBMIX),INDICE(NNAM),IXSFL(NNAM+1)
      REAL              XSSCAT(NBMIX,NGROUP,NGROUP,NL),
     >                  SADXS(0:NBMIX,NGROUP,NNAM),
     >                  PCHI(NBMIX,NIFISS,NGROUP),
     >                  PFIS(NBMIX,NIFISS,NGROUP),
     >                  SASCAT(NBMIX,NGCOND,NGROUP,NL)
      CHARACTER         HVECT(NEDMAC)*6,NAME(NNAM)*6
      DOUBLE PRECISION  RATECM(0:NMERGE,NGCOND,NTAUXT),
     >                  SCATTD(NMERGE,NGCOND,NGCOND,NL),CHISUM(NIFISS),
     >                  DASCAT(NBMIX,NGCOND,NGROUP,NL)
C----
C  LOCAL PARAMETERS
C----
      CHARACTER         CM*2,CGRPNM*12,IGCAR*2
      INTEGER           INAM,IGROUP,
     >                  IMIX,IFIS,IMRG,IREG,IGRC,IGRFIN,IGRDEB,
     >                  ID,FL,IGR,IL,JGRC,JGROUP,JGRFIN,JGRDEB
      DOUBLE PRECISION  DFACT,ACCUMU
C-----
C ADJOINT SOURCE COMPUTING
C-----
      DO 100 INAM=1,NNAM
       IF(INDICE(INAM).GT.0 .AND. IXSFL(INAM).EQ.1) THEN
        IGRFIN=0
        DO 20 IGRC=1,NGCOND
         IGRDEB=IGRFIN+1
         IGRFIN=IGCOND(IGRC)
         DO 30 IGROUP=IGRDEB,IGRFIN
            CALL XDRSET(FLAGMX,NBMIX,0)
            DO 40 IMRG=1,NMERGE
              DO 50 IREG=1,NREGIO
                IMIX=MATCOD(IREG)
                ID=INDICE(INAM)
                FL=FLAGMX(IMIX)
                IF(IMERGE(IREG).EQ.IMRG.AND.IMIX.GT.0.AND.FL.EQ.0) THEN
                 FLAGMX(IMIX)=1
                 IF (INAM.EQ.5) THEN
C----
C FISSION SPECTRUM
C----
                  IF(RATECM(IMRG,IGRC,4).GT.0.0) THEN
                   DFACT=1.0/RATECM(IMRG,IGRC,4)
                   DO 51 IFIS=1,NIFISS
                    CHISUM(IFIS)=0.D0
                    DO 52 IGR=IGRDEB,IGRFIN
                     CHISUM(IFIS)=CHISUM(IFIS)+PCHI(IMIX,IFIS,IGR)
 52                 CONTINUE
 51                CONTINUE
                   ACCUMU=0.D0
                   DO 53 IFIS=1,NIFISS
                    ACCUMU=ACCUMU+CHISUM(IFIS)*PFIS(IMIX,IFIS,IGROUP)
 53                CONTINUE
                   SADXS(IMIX,IGROUP,INAM)=DFACT*(ACCUMU-
     >              RATECM(IMRG,IGRC,ID)*SADXS(IMIX,IGROUP,3))
                  ENDIF
                 ELSE IF(INAM.EQ.3 .OR. INAM.EQ.4) THEN
C----
C NUSIGF AND NFTOT XS
C----
                  ACCUMU=0.D0
                  DO 54 IFIS=1,NIFISS
                    ACCUMU=ACCUMU+RATECM(IMRG,IGRC,ID+IFIS)
 54               CONTINUE
                  DFACT=1.0/RATECM(IMRG,IGRC,1)
                  SADXS(IMIX,IGROUP,INAM)=
     >                         DFACT*SADXS(IMIX,IGROUP,INAM)
     >                        -DFACT**2*ACCUMU
                 ELSE
C----
C STANDARD XS
C----
                  DFACT=1.0/RATECM(IMRG,IGRC,1)
                  SADXS(IMIX,IGROUP,INAM)=
     >                         DFACT*SADXS(IMIX,IGROUP,INAM)
     >                        -DFACT**2*RATECM(IMRG,IGRC,ID)
                 ENDIF
                ENDIF
 50           CONTINUE
 40         CONTINUE
 30      CONTINUE
 20     CONTINUE
       ENDIF
 100  CONTINUE
C-----
C COMPRESSED GENERALIZED ADJOINT SOURCES PRINTING
C----
      DO 400 INAM=1,NNAM
       IF(INDICE(INAM).GT.0 .AND. IXSFL(INAM).EQ.1) THEN
        DO 60 IGROUP=1,NGROUP
         WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
         CALL LCMSIX(IPTMP,CGRPNM,ILCMUP)
         CALL LCMPUT(IPTMP,NAME(INAM),NBMIX,2,SADXS(1,IGROUP,INAM))
         CALL LCMSIX(IPTMP,CGRPNM,ILCMDN)
 60     CONTINUE
       ENDIF
 400  CONTINUE
C---
C SCATTERING ADJOINT SOURCE  INITIALIZATION
C---
      IF(IXSFL(NNAM+1).EQ.1) THEN
       IL=1
       CALL XDRSET(SASCAT,NBMIX*NGCOND*NGROUP*NL,0.0)
       CALL XDRDBL(DASCAT,NBMIX*NGCOND*NGROUP*NL,0.0D0)
c$$$       DO 140 IL=1,NL
        DO 141 JGROUP=1,NGROUP
         DO 142 IMIX=1,NBMIX
           IGRFIN=0
           DO 143 IGRC=1,NGCOND
             IGRDEB=IGRFIN+1
             IGRFIN=IGCOND(IGRC)
             ACCUMU=0.D0
             DO 144 IGROUP=IGRDEB,IGRFIN
                ACCUMU=ACCUMU+XSSCAT(IMIX,IGROUP,JGROUP,IL)
 144         CONTINUE
             DASCAT(IMIX,IGRC,JGROUP,IL)=ACCUMU
 143       CONTINUE
 142      CONTINUE
 141    CONTINUE
c$$$ 140   CONTINUE
C---
C SCATTERING ADJOINT SOURCE  COMPUTATION LOOP
C---
c$$$       DO 189 IL=1,NL
        DO 190 IGRC=1,NGCOND
            CALL XDRSET(FLAGMX,NBMIX,0)
            DO 192 IMRG=1,NMERGE
              DO 193 IREG=1,NREGIO
                IMIX=MATCOD(IREG)
                FL=FLAGMX(IMIX)
                IF(IMERGE(IREG).EQ.IMRG.AND.IMIX.GT.0.AND.FL.EQ.0) THEN
                 FLAGMX(IMIX)=1
                 JGRFIN=0
                 DO 194 JGRC=1,NGCOND
                  JGRDEB=JGRFIN+1
                  JGRFIN=IGCOND(JGRC)
                  DFACT=1.0/RATECM(IMRG,JGRC,1)
                  DO 195 JGROUP=JGRDEB,JGRFIN
                    SASCAT(IMIX,IGRC,JGROUP,IL)=
     >                DFACT*DASCAT(IMIX,IGRC,JGROUP,IL)
     >               -DFACT**2*SCATTD(IMRG,IGRC,JGRC,IL)
 195              CONTINUE
 194             CONTINUE
                ENDIF
 193          CONTINUE
 192        CONTINUE
 190    CONTINUE
c$$$ 189   CONTINUE
C------
C COMPRESSED SCATTERING ADJOINT SOURCE PRINTING
C------
       DO 201 IGRC=1,NGCOND
        WRITE (IGCAR,'(I2)') IGRC
c$$$        DO 202 IL=1,NL
          WRITE (CM,'(I2)') IL-1
          DO 203 JGROUP=1,NGROUP
           WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') JGROUP,NGROUP
           CALL LCMSIX(IPTMP,CGRPNM,ILCMUP)
           CALL LCMPUT(IPTMP,'SCAD'//CM//IGCAR,NBMIX,2,
     >                            SASCAT(1,IGRC,JGROUP,IL))
           CALL LCMSIX(IPTMP,CGRPNM,ILCMDN)
 203      CONTINUE
c$$$ 202    CONTINUE
 201   CONTINUE
      ENDIF
      RETURN
      END
