*DECK SYBALP
      SUBROUTINE SYBALP (NPIJ,MAXPTS,Y,SIGT,PIJ,ALBG,ALBD,AUXI,
     >                   X,IND,INDV)
*
*----------------------------------------------------------------------
*
* PIJ CALCULATION USING THE METHOD OF KAVENOKY IN SLAB GEOMETRY.
*
* INPUT PARAMETERS:
*    NPIJ   : NUMBER OF REGIONS.
*    MAXPTS : FIRST DIMENSION OF MATRIX PIJ.
*    Y      : ABSCISSA ARRAY. DIMENSION Y(NPIJ+1)
*    SIGT   : TOTAL CROSS SECTION ARRAY. DIMENSION SIGT(NPIJ)
*    ALBG   : LEFT ALBEDO.
*    ALBD   : RIGHT ALBEDO.
*    AUXI   : DUMMY WORK ARRAY. DIMENSION AUXI(4*NPIJ)
*    X      : DUMMY VECTOR OF LENGTH NPIJ+1
*    IND    : DUMMY VECTOR OF LENGTH NPIJ+1
*    INDV   : DUMMY VECTOR OF LENGTH NPIJ+1
*
* OUTPUT PARAMETERS:
*    PIJ    : REDUCED COLLISION PROBABILITIES ARRAY.
*             DIMENSION  PIJ(MAXPTS,NPIJ)
*
* REFERENCE:
* A. KAVENOKY, 'CALCUL ET UTILISATION DES PROBABILITES DE PREMIERE
* COLLISION POUR LES MILIEUX HETEROGENES A UNE DIMENSION: LES PROGRAMMES
* ALCOLL ET CORTINA', NOTE CEA-N-1077, COMMISSARIAT A L'ENERGIE
* ATOMIQUE, MARS 1969.
*
*----------------------------------------------------------------------
*
      PARAMETER (TRONC=10.0)
      DIMENSION Y(*),SIGT(*),PIJ(MAXPTS,*),AUXI(*),X(NPIJ+1),
     >          IND(NPIJ+1),INDV(NPIJ+1)
      CHARACTER HA*40,HB(2)*33
      DATA HA/'ERROR IN CALLING ARGUMENTS OF SYBALP'/
      DATA HB/'R(I+1) IS LESS THAN R(I)         ',
     1        'A CROSS SECTION IS NEGATIVE      '/
*
      DO 10 J=1,NPIJ
      IF ((Y(J+1)-Y(J)).LT.0.) CALL XABORT('SYBALP: '//HA//HB(1))
      IF (SIGT(J).LT.0.) CALL XABORT('SYBALP: '//HA//HB(2))
   10 CONTINUE
*
*     AUXI(IM)= 0.0 FOR A VOID ZONE
*               1.0 FOR A FILLED ZONE
*
      K=0
      DO 120 IM=1,NPIJ
         IND(IM)=1
         IF(SIGT(IM).LT.1.E-6)IND(IM)=0
         AUXI(IM)=IND(IM)
         IF(IND(IM))121,121,120
  121    K=K+1
         INDV(K)=IM
  120 CONTINUE
*
*     KMAX IS THE NUMBER OF VOID ZONES
*
      KMAX=K
*
*     INITIALIZATION OF "PIJ" MATRIX AND OF MFP VECTOR X()
*
      X(1)=0.0
      DO 1 I=1,NPIJ
         X(I+1)=X(I)+SIGT(I)*(Y(I+1)-Y(I))
         DO 1 J=1,NPIJ
            PIJ(I,J)=0.0
    1    CONTINUE
*
*     TAU IS THE TOTAL MFP FOR THE SLAB
*
      TAU=X(NPIJ+1)
      N1=NPIJ+1
      EPS1=1.
      EPS2=1.
*
*      "E3" FUNCTIONS ARE USED.
*
      NA=3
      DO 2 I=1,NPIJ
         DO 2 J=I,N1
            S=X(I)
            T=X(J)
*
            U=T-S
            IF (U.LT.0.) U=0.
            EXPN= TABEN(NA,U)
            EXPIN=EXPN
            IF((ALBG.LT.1.E-04).AND.(ALBD.LT.1.E-04)) GO TO 250
            IF (ALBD.LT.1.E-4) GO TO 201
            IF (ALBG.LT.1.E-4) GO TO 202
            BETA=1.
  203       BETA=BETA*ALBG*ALBD
            U=U+2.*TAU
            IF (U.GT.TRONC) GO TO 204
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN+BETA*EXPN
            GO TO 203
  204       BETA=ALBG
            U=S+T
  205       IF (U.GT.TRONC) GO TO 206
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-BETA*EXPN*EPS1*EPS2
            BETA=BETA*ALBG*ALBD
            U=U+2.*TAU
            GO TO 205
  206       BETA=ALBD
            U=-S-T
  207       U=U+2.*TAU
            IF (U.LT.0.) U=0.
            IF (U.GT.TRONC) GO TO 208
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-BETA*EXPN*EPS1
            BETA=BETA*ALBG*ALBD
            GO TO 207
  208       BETA=1.
            U=(S-T)*EPS2
  209       U=U+2.*TAU
            IF (U.GT.TRONC) GO TO 250
            BETA=BETA*ALBG*ALBD
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN+BETA*EXPN*EPS2
            GO TO 209
  201       U=S+T
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-EXPN*ALBG*EPS1*EPS2
            GO TO 250
  202       U=2.*TAU-S-T
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-EXPN*ALBD*EPS1
*
  250       ZT=EXPIN
            IF (J.EQ.1) GO TO 30
            IF (I.EQ.1) GO TO 31
            PIJ(I-1,J-1)=PIJ(I-1,J-1)-ZT*AUXI(I-1)*AUXI(J-1)
            IF (J.EQ.N1) GO TO 31
            PIJ(I-1,J)=PIJ(I-1,J)+ZT*AUXI(I-1)*AUXI(J)
   31       A=1.
            IF (J-I-1) 30,32,33
   32       A=2.
   33       PIJ(I,J-1)=PIJ(I,J-1)+ZT*AUXI(I)*AUXI(J-1)*A
   30       IF (J.EQ.N1) GO TO 2
            PIJ(I,J)=PIJ(I,J)-ZT*AUXI(I)*AUXI(J)
    2 CONTINUE
      S=X(NPIJ+1)
      T=X(NPIJ+1)
*
      U=T-S
      IF (U.LT.0.) U=0.
      EXPN= TABEN(NA,U)
      EXPIN=EXPN
      IF((ALBG.LT.1.E-04).AND.(ALBD.LT.1.E-04)) GO TO 350
      IF (ALBD.LT.1.E-4) GO TO 301
      IF (ALBG.LT.1.E-4) GO TO 302
      BETA=1.
  303 BETA=BETA*ALBG*ALBD
      U=U+2.*TAU
      IF (U.GT.TRONC) GO TO 304
      EXPN= TABEN(NA,U)
      EXPIN=EXPIN+BETA*EXPN
      GO TO 303
  304 BETA=ALBG
      U=S+T
  305 IF (U.GT.TRONC) GO TO 306
      EXPN= TABEN(NA,U)
      EXPIN=EXPIN-BETA*EXPN*EPS1*EPS2
      BETA=BETA*ALBG*ALBD
      U=U+2.*TAU
      GO TO 305
  306 BETA=ALBD
      U=-S-T
  307 U=U+2.*TAU
      IF (U.LT.0.) U=0.
      IF (U.GT.TRONC) GO TO 308
      EXPN= TABEN(NA,U)
      EXPIN=EXPIN-BETA*EXPN*EPS1
      BETA=BETA*ALBG*ALBD
      GO TO 307
  308 BETA=1.
      U=(S-T)*EPS2
  309 U=U+2.*TAU
      IF (U.GT.TRONC) GO TO 350
      BETA=BETA*ALBG*ALBD
      EXPN= TABEN(NA,U)
      EXPIN=EXPIN+BETA*EXPN*EPS2
      GO TO 309
  301 U=S+T
      EXPN= TABEN(NA,U)
      EXPIN=EXPIN-EXPN*ALBG*EPS1*EPS2
      GO TO 350
  302 U=2.*TAU-S-T
      EXPN= TABEN(NA,U)
      EXPIN=EXPIN-EXPN*ALBD*EPS1
*
  350 PIJ(NPIJ,NPIJ)=PIJ(NPIJ,NPIJ)-AUXI(NPIJ)*EXPIN
*
*     IF(KMAX.EQ.0)THEN THERE IS NO VOID ZONE, AND WE RENORMALIZE ONLY
*
      IF(KMAX.EQ.0)GO TO 40
*
*     NOW WE COMPUTE CONTRIBUTION TO VOID ZONES FROM NON-VOID ONES
*
      EPS2=-1.
*
*      "E2" FUNCTIONS ARE USED.
*
      NA=2
      DO 12 K=1,KMAX
         I=INDV(K)
         IF(I.EQ.1)GO TO 1000
         EPS1=-1.
         DO 21 J=1,I
            S=X(J)
            T=X(I)
*
            U=T-S
            IF (U.LT.0.) U=0.
            EXPN= TABEN(NA,U)
            EXPIN=EXPN
            IF((ALBG.LT.1.E-04).AND.(ALBD.LT.1.E-04)) GO TO 450
            IF (ALBD.LT.1.E-4) GO TO 401
            IF (ALBG.LT.1.E-4) GO TO 402
            BETA=1.
  403       BETA=BETA*ALBG*ALBD
            U=U+2.*TAU
            IF (U.GT.TRONC) GO TO 404
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN+BETA*EXPN
            GO TO 403
  404       BETA=ALBG
            U=S+T
  405       IF (U.GT.TRONC) GO TO 406
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-BETA*EXPN*EPS1*EPS2
            BETA=BETA*ALBG*ALBD
            U=U+2.*TAU
            GO TO 405
  406       BETA=ALBD
            U=-S-T
  407       U=U+2.*TAU
            IF (U.LT.0.) U=0.
            IF (U.GT.TRONC) GO TO 408
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-BETA*EXPN*EPS1
            BETA=BETA*ALBG*ALBD
            GO TO 407
  408       BETA=1.
            U=(S-T)*EPS2
  409       U=U+2.*TAU
            IF (U.GT.TRONC) GO TO 450
            BETA=BETA*ALBG*ALBD
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN+BETA*EXPN*EPS2
            GO TO 409
  401       U=S+T
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-EXPN*ALBG*EPS1*EPS2
            GO TO 450
  402       U=2.*TAU-S-T
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-EXPN*ALBD*EPS1
*
  450       ZT=EXPIN
            IF(J.EQ.1)GO TO 22
            PIJ(J-1,I)=PIJ(J-1,I)+ZT*AUXI(J-1)
            IF(J.EQ.I )GO TO 21
   22       PIJ(J,I)=PIJ(J,I)-ZT*AUXI(J)
   21    CONTINUE
 1000    I1=I+1
         EPS1=1.
         DO 23 J=I1,N1
            S=X(I1)
            T=X(J)
*
            U=T-S
            IF (U.LT.0.) U=0.
            EXPN= TABEN(NA,U)
            EXPIN=EXPN
            IF((ALBG.LT.1.E-04).AND.(ALBD.LT.1.E-04)) GO TO 550
            IF (ALBD.LT.1.E-4) GO TO 501
            IF (ALBG.LT.1.E-4) GO TO 502
            BETA=1.
  503       BETA=BETA*ALBG*ALBD
            U=U+2.*TAU
            IF (U.GT.TRONC) GO TO 504
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN+BETA*EXPN
            GO TO 503
  504       BETA=ALBG
            U=S+T
  505       IF (U.GT.TRONC) GO TO 506
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-BETA*EXPN*EPS1*EPS2
            BETA=BETA*ALBG*ALBD
            U=U+2.*TAU
            GO TO 505
  506       BETA=ALBD
            U=-S-T
  507       U=U+2.*TAU
            IF (U.LT.0.) U=0.
            IF (U.GT.TRONC) GO TO 508
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-BETA*EXPN*EPS1
            BETA=BETA*ALBG*ALBD
            GO TO 507
  508       BETA=1.
            U=(S-T)*EPS2
  509       U=U+2.*TAU
            IF (U.GT.TRONC) GO TO 550
            BETA=BETA*ALBG*ALBD
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN+BETA*EXPN*EPS2
            GO TO 509
  501       U=S+T
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-EXPN*ALBG*EPS1*EPS2
            GO TO 550
  502       U=2.*TAU-S-T
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-EXPN*ALBD*EPS1
*
  550       ZT=EXPIN
            IF(J.EQ.N1) GO TO 24
            PIJ(I,J)=PIJ(I,J)+ZT*AUXI(J)
            IF(J.EQ.I1)  GO TO 23
   24       PIJ(I,J-1)=PIJ(I,J-1)-ZT*AUXI(J-1)
   23    CONTINUE
   12 CONTINUE
      IF( KMAX.EQ.1 )GO TO 40
*
*     NOW WE COMPUTE CONTRIBUTION TO VOID ZONES FROM NON-VOID ONES
*
      EPS1=1.
      EPS2=1.
*
*      "E1" FUNCTIONS ARE USED.
*
      NA=1
      DO 4 KI=1,KMAX
         I= INDV(KI)
         PIJ(I,I)= TABEN(NA,0.0)
         DO 4 KJ=KI+1,KMAX
            J= INDV(KJ)
            S=X(I)
            T=X(J)
*
            U=T-S
            IF (U.LT.0.) U=0.
            EXPN= TABEN(NA,U)
            EXPIN=EXPN
            IF((ALBG.LT.1.E-04).AND.(ALBD.LT.1.E-04)) GO TO 650
            IF (ALBD.LT.1.E-4) GO TO 601
            IF (ALBG.LT.1.E-4) GO TO 602
            BETA=1.
  603       BETA=BETA*ALBG*ALBD
            U=U+2.*TAU
            IF (U.GT.TRONC) GO TO 604
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN+BETA*EXPN
            GO TO 603
  604       BETA=ALBG
            U=S+T
  605       IF (U.GT.TRONC) GO TO 606
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-BETA*EXPN*EPS1*EPS2
            BETA=BETA*ALBG*ALBD
            U=U+2.*TAU
            GO TO 605
  606       BETA=ALBD
            U=-S-T
  607       U=U+2.*TAU
            IF (U.LT.0.) U=0.
            IF (U.GT.TRONC) GO TO 608
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-BETA*EXPN*EPS1
            BETA=BETA*ALBG*ALBD
            GO TO 607
  608       BETA=1.
            U=(S-T)*EPS2
  609       U=U+2.*TAU
            IF (U.GT.TRONC) GO TO 650
            BETA=BETA*ALBG*ALBD
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN+BETA*EXPN*EPS2
            GO TO 609
  601       U=S+T
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-EXPN*ALBG*EPS1*EPS2
            GO TO 650
  602       U=2.*TAU-S-T
            EXPN= TABEN(NA,U)
            EXPIN=EXPIN-EXPN*ALBD*EPS1
*
  650       ZT=EXPIN
            PIJ(I,J)=PIJ(I,J) + ZT
    4 CONTINUE
*
*     RENORMALISATION OF "PIJ" ACCORDING TO CEA-1076
*      THE "PIJ" MATRIX IS GIVEN IN THE REDUCED SENSE.
*
   40 DO 100 I=1,NPIJ
         IF(IND(I).EQ.0)GO TO 102
         PIJ(I,I)=(1.+PIJ(I,I)/2./(X(I+1)-X(I)))/SIGT(I)
         IF(I.EQ.NPIJ)RETURN
  102    I1=I+1
         AUXK=SIGT(I)
         IF (IND(I).EQ.0) AUXK=1./(Y(I+1)-Y(I))
         DO 700 J=I1,NPIJ
            AUXJ=SIGT(J)
            IF (IND(J).EQ.0) AUXJ=1./(Y(J+1)-Y(J))
            PIJ(I,J)=PIJ(I,J)/2./AUXK/AUXJ/(Y(I+1)-Y(I))
            PIJ(J,I)=PIJ(I,J)*(Y(I+1)-Y(I))/(Y(J+1)-Y(J))
  700    CONTINUE
  100 CONTINUE
      RETURN
      END
